"""Discord通知機能のテスト"""

import os
import json
import pytest
import asyncio
from pathlib import Path
from unittest.mock import Mock, patch, AsyncMock, MagicMock, create_autospec
from src.infrastructure.notifiers.discord_notifier import DiscordNotifier
from src.infrastructure.config.config_manager import ConfigManager


class TestDiscordNotifier:
    """DiscordNotifierのテスト"""
    
    @pytest.fixture
    def mock_config_manager(self):
        """モックのConfigManager"""
        mock_cm = Mock(spec=ConfigManager)
        mock_cm.get_notification_settings.return_value = {
            'discord': {
                'enabled': True,
                'webhook_url': 'https://discord.com/api/webhooks/test/webhook',
                'thread_id': '123456789',
                'thread_name': 'test-thread',
                'mention_everyone': False
            }
        }
        return mock_cm
    
    @pytest.fixture
    def notifier(self, mock_config_manager):
        """テスト用のDiscordNotifierインスタンス"""
        return DiscordNotifier(config_manager=mock_config_manager)
    
    def test_init(self, notifier):
        """初期化のテスト"""
        assert notifier.agent_name is not None
        assert notifier.session_id is not None
        assert notifier.config is not None
    
    def test_load_config(self, notifier):
        """設定読み込みのテスト"""
        config = notifier.config
        assert config['DISCORD_WEBHOOK_URL'] == 'https://discord.com/api/webhooks/test/webhook'
        assert config['DISCORD_THREAD_ID'] == '123456789'
        assert config['THREAD_NAME'] == 'test-thread'
        assert config['DISCORD_MENTION_EVERYONE'] == 'false'
    
    def test_load_config_no_settings(self):
        """設定がない場合のテスト"""
        mock_cm = Mock(spec=ConfigManager)
        mock_cm.get_notification_settings.return_value = {}
        notifier = DiscordNotifier(config_manager=mock_cm)
        # デフォルト値が設定される
        assert notifier.config.get('THREAD_NAME') == 'general'
    
    def test_load_config_with_env_expansion(self):
        """環境変数展開のテスト"""
        with patch.dict(os.environ, {'DISCORD_WEBHOOK_URL': 'https://test.webhook'}):
            mock_cm = Mock(spec=ConfigManager)
            mock_cm.get_notification_settings.return_value = {
                'discord': {
                    'webhook_url': '${DISCORD_WEBHOOK_URL}',
                    'thread_id': '999',
                    'mention_everyone': True
                }
            }
            notifier = DiscordNotifier(config_manager=mock_cm)
            assert notifier.config['DISCORD_WEBHOOK_URL'] == 'https://test.webhook'
            assert notifier.config['DISCORD_THREAD_ID'] == '999'
            assert notifier.config['DISCORD_MENTION_EVERYONE'] == 'true'
    
    def test_get_session_id_from_session_manager(self, mock_config_manager):
        """SessionManagerからセッションID取得のテスト"""
        mock_session_manager = Mock()
        mock_session_manager.session_id = 'test-session-123'
        mock_session_manager.agent_name = 'Test-Agent'

        with patch('src.infrastructure.notifiers.discord_notifier.get_session_manager', return_value=mock_session_manager):
            notifier = DiscordNotifier(config_manager=mock_config_manager)
            assert notifier.session_id == 'test-session-123'
    
    def test_get_session_id_fallback(self, mock_config_manager):
        """セッションIDのフォールバックテスト"""
        with patch.dict(os.environ, {}, clear=True):
            notifier = DiscordNotifier(config_manager=mock_config_manager)
            assert notifier.session_id == str(os.getpid())
    
    def test_get_agent_name_from_session_manager(self, mock_config_manager):
        """SessionManagerからエージェント名取得のテスト"""
        mock_session_manager = Mock()
        mock_session_manager.session_id = 'test-session'
        mock_session_manager.agent_name = 'Mochi-1234'

        with patch('src.infrastructure.notifiers.discord_notifier.get_session_manager', return_value=mock_session_manager):
            notifier = DiscordNotifier(config_manager=mock_config_manager)
            assert notifier.agent_name == 'Mochi-1234'
    
    @pytest.mark.asyncio
    async def test_send_message_success(self, notifier):
        """メッセージ送信成功のテスト"""
        with patch('src.infrastructure.notifiers.discord_notifier.aiohttp') as mock_aiohttp:
            # Mock レスポンス
            mock_response = AsyncMock()
            mock_response.status = 204
            
            # Mock post context manager
            class MockPostContext:
                async def __aenter__(self):
                    return mock_response
                async def __aexit__(self, *args):
                    return None
            
            # Mock session
            mock_session = AsyncMock()
            mock_session.post = lambda *args, **kwargs: MockPostContext()
            
            # Mock ClientSession context manager
            class MockClientSession:
                def __init__(self, *args, **kwargs):
                    pass
                async def __aenter__(self):
                    return mock_session
                async def __aexit__(self, *args):
                    return None
            
            mock_aiohttp.ClientSession = MockClientSession
            
            result = await notifier.send_message("test message")
            
            assert result['success'] is True
            assert result['message'] == "test message"
            assert result['agent_name'] == notifier.agent_name
    
    @pytest.mark.asyncio
    async def test_send_message_with_thread(self, notifier):
        """スレッドIDを指定した送信のテスト"""
        with patch('src.infrastructure.notifiers.discord_notifier.aiohttp') as mock_aiohttp:
            mock_response = AsyncMock()
            mock_response.status = 204
            
            class MockPostContext:
                async def __aenter__(self):
                    return mock_response
                async def __aexit__(self, *args):
                    return None
            
            mock_session = AsyncMock()
            mock_session.post = Mock(return_value=MockPostContext())
            
            class MockClientSession:
                def __init__(self, *args, **kwargs):
                    pass
                async def __aenter__(self):
                    return mock_session
                async def __aexit__(self, *args):
                    return None
            
            mock_aiohttp.ClientSession = MockClientSession
            
            result = await notifier.send_message(
                "test message",
                thread_id="999888777"
            )
            
            # URLにthread_idが含まれることを確認
            call_args = mock_session.post.call_args
            assert "thread_id=999888777" in call_args[0][0]
    
    @pytest.mark.asyncio
    async def test_send_message_with_mention(self, notifier):
        """@everyoneメンション付き送信のテスト"""
        with patch('src.infrastructure.notifiers.discord_notifier.aiohttp') as mock_aiohttp:
            mock_response = AsyncMock()
            mock_response.status = 204
            
            class MockPostContext:
                async def __aenter__(self):
                    return mock_response
                async def __aexit__(self, *args):
                    return None
            
            mock_session = AsyncMock()
            mock_session.post = Mock(return_value=MockPostContext())
            
            class MockClientSession:
                def __init__(self, *args, **kwargs):
                    pass
                async def __aenter__(self):
                    return mock_session
                async def __aexit__(self, *args):
                    return None
            
            mock_aiohttp.ClientSession = MockClientSession
            
            result = await notifier.send_message(
                "test message",
                mention_everyone=True
            )
            
            # ペイロードに@everyoneが含まれることを確認
            call_args = mock_session.post.call_args
            payload = call_args[1]['json']
            assert '@everyone' in payload['content']
            assert payload['allowed_mentions'] == {'parse': ['everyone']}
    
    @pytest.mark.asyncio
    async def test_send_message_no_webhook(self):
        """Webhook URLが設定されていない場合のテスト"""
        mock_cm = Mock(spec=ConfigManager)
        mock_cm.get_notification_settings.return_value = {'discord': {}}
        notifier = DiscordNotifier(config_manager=mock_cm)
        result = await notifier.send_message("test message")
        
        assert result['success'] is False
        assert 'not configured' in result['error']
    
    @pytest.mark.asyncio
    async def test_send_message_http_error(self, notifier):
        """HTTP エラーのテスト"""
        with patch('src.infrastructure.notifiers.discord_notifier.aiohttp') as mock_aiohttp:
            # Mock レスポンス
            mock_response = AsyncMock()
            mock_response.status = 500
            mock_response.text = AsyncMock(return_value="Internal Server Error")
            
            # Mock post context manager
            class MockPostContext:
                async def __aenter__(self):
                    return mock_response
                async def __aexit__(self, *args):
                    return None
            
            # Mock session
            mock_session = AsyncMock()
            mock_session.post = lambda *args, **kwargs: MockPostContext()
            
            # Mock ClientSession context manager
            class MockClientSession:
                def __init__(self, *args, **kwargs):
                    pass
                async def __aenter__(self):
                    return mock_session
                async def __aexit__(self, *args):
                    return None
            
            mock_aiohttp.ClientSession = MockClientSession
            
            result = await notifier.send_message("test message")
            
            assert result['success'] is False
            assert "HTTP 500" in result['error']
    
    @pytest.mark.asyncio
    async def test_send_message_exception(self, notifier):
        """例外発生時のテスト"""
        with patch('src.infrastructure.notifiers.discord_notifier.aiohttp') as mock_aiohttp:
            mock_aiohttp.ClientSession.side_effect = Exception("Connection error")
            
            result = await notifier.send_message("test message")
            
            assert result['success'] is False
            assert "Connection error" in result['error']
    
    def test_send_sync(self, notifier):
        """同期送信のテスト"""
        # AsyncMockを使う
        with patch.object(notifier, 'send_message', new_callable=AsyncMock) as mock_send:
            mock_send.return_value = {
                'success': True,
                'message': 'test',
                'agent_name': 'test-agent'
            }
            
            # 同期実行
            result = notifier.send_sync("test message")
            
            assert result['success'] is True
            assert result['message'] == 'test'
            assert result['agent_name'] == 'test-agent'
    
    def test_main_success(self, mock_config_manager, capsys):
        """CLIメイン関数の成功テスト"""
        with patch('sys.argv', ['discord_notifier.py', 'Test notification']):
            with patch.object(DiscordNotifier, 'send_sync') as mock_send:
                mock_send.return_value = {
                    'success': True,
                    'message': 'Test notification',
                    'agent_name': 'test-agent'
                }
                
                with pytest.raises(SystemExit) as excinfo:
                    from src.infrastructure.notifiers.discord_notifier import main
                    main()
                
                assert excinfo.value.code == 0
                captured = capsys.readouterr()
                assert "Message sent to Discord" in captured.err
    
    def test_main_failure(self, mock_config_manager, capsys):
        """CLIメイン関数の失敗テスト"""
        with patch('sys.argv', ['discord_notifier.py', 'Test notification']):
            with patch.object(DiscordNotifier, 'send_sync') as mock_send:
                mock_send.return_value = {
                    'success': False,
                    'error': 'Connection failed'
                }
                
                with pytest.raises(SystemExit) as excinfo:
                    from src.infrastructure.notifiers.discord_notifier import main
                    main()
                
                assert excinfo.value.code == 0
                captured = capsys.readouterr()
                assert "Failed to send Discord message" in captured.err


"""hook_executor.py のテスト"""

import pytest
import json
import subprocess
from pathlib import Path
from unittest.mock import patch, MagicMock, Mock

from src.infrastructure.hooks.hook_executor import HookExecutor


class TestHookExecutorInit:
    """HookExecutor初期化のテスト"""

    def test_init_creates_hook_manager(self):
        """初期化時にHookManagerが作成される"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager') as mock_manager:
            executor = HookExecutor()
            mock_manager.assert_called_once()

    def test_init_registers_builtin_hooks(self):
        """初期化時に内蔵フックが登録される"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager'):
            executor = HookExecutor()
            assert 'implementation_design' in executor.builtin_hooks

    def test_init_with_claude_dir(self):
        """claude_dirを指定して初期化できる"""
        claude_dir = Path('/tmp/test_claude')
        with patch('src.infrastructure.hooks.hook_executor.HookManager') as mock_manager:
            executor = HookExecutor(claude_dir=claude_dir)
            mock_manager.assert_called_once_with(claude_dir)


class TestMatchesTool:
    """_matches_tool メソッドのテスト"""

    def test_empty_matcher_matches_all(self):
        """空文字列マッチャーは全てにマッチ"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager'):
            executor = HookExecutor()
            assert executor._matches_tool('Edit', '') is True
            assert executor._matches_tool('Write', '') is True

    def test_exact_match(self):
        """完全一致でマッチ"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager'):
            executor = HookExecutor()
            assert executor._matches_tool('Edit', 'Edit') is True
            assert executor._matches_tool('Edit', 'Write') is False

    def test_pipe_separator_or_match(self):
        """パイプ区切りでOR条件マッチ"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager'):
            executor = HookExecutor()
            assert executor._matches_tool('Edit', 'Edit|Write') is True
            assert executor._matches_tool('Write', 'Edit|Write') is True
            assert executor._matches_tool('Read', 'Edit|Write') is False

    def test_wildcard_match(self):
        """ワイルドカードでマッチ"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager'):
            executor = HookExecutor()
            assert executor._matches_tool('mcp__serena__create', 'mcp__serena__*') is True
            assert executor._matches_tool('mcp__other__create', 'mcp__serena__*') is False


class TestMatchesSinglePattern:
    """_matches_single_pattern メソッドのテスト"""

    def test_exact_match(self):
        """完全一致"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager'):
            executor = HookExecutor()
            assert executor._matches_single_pattern('Edit', 'Edit') is True
            assert executor._matches_single_pattern('Edit', 'Write') is False

    def test_wildcard_asterisk(self):
        """アスタリスクワイルドカード"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager'):
            executor = HookExecutor()
            assert executor._matches_single_pattern('mcp__test__func', 'mcp__*') is True
            assert executor._matches_single_pattern('mcp__test__func', '*__func') is True
            assert executor._matches_single_pattern('mcp__test__func', 'mcp__test__*') is True

    def test_no_wildcard_exact_match_only(self):
        """ワイルドカードなしの場合は完全一致のみ"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager'):
            executor = HookExecutor()
            # ?はワイルドカードとして扱われない（*のみ対応）
            assert executor._matches_single_pattern('Edit', 'Edi?') is False
            assert executor._matches_single_pattern('Edit', 'Edit') is True


class TestExecuteSingleHook:
    """_execute_single_hook メソッドのテスト"""

    def test_non_command_type_returns_none(self):
        """type='command'以外はNoneを返す"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager'):
            executor = HookExecutor()
            result = executor._execute_single_hook(
                {'type': 'script'}, 'PreToolUse', 'Edit', {}
            )
            assert result is None

    def test_missing_command_returns_none(self):
        """commandがない場合はNoneを返す"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager'):
            executor = HookExecutor()
            result = executor._execute_single_hook(
                {'type': 'command'}, 'PreToolUse', 'Edit', {}
            )
            assert result is None

    def test_successful_command_execution(self):
        """コマンド実行成功時に結果を返す"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager'):
            executor = HookExecutor()

            mock_result = MagicMock()
            mock_result.stdout = json.dumps({'decision': 'allow', 'reason': 'test'})

            with patch('subprocess.run', return_value=mock_result):
                result = executor._execute_single_hook(
                    {'type': 'command', 'command': 'echo test'},
                    'PreToolUse', 'Edit', {'file_path': '/test.py'}
                )

            assert result['decision'] == 'allow'
            assert result['reason'] == 'test'

    def test_command_with_session_id(self):
        """セッションIDが入力データに含まれる"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager'):
            executor = HookExecutor()

            mock_result = MagicMock()
            mock_result.stdout = json.dumps({'decision': 'allow'})

            with patch('subprocess.run', return_value=mock_result) as mock_run:
                executor._execute_single_hook(
                    {'type': 'command', 'command': 'echo test'},
                    'PreToolUse', 'Edit', {}, session_id='test-session'
                )

            # 入力データにsession_idが含まれることを確認
            call_args = mock_run.call_args
            input_data = json.loads(call_args.kwargs['input'])
            assert input_data['session_id'] == 'test-session'

    def test_invalid_json_response(self):
        """無効なJSON応答時はエラーを含む結果を返す"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager'):
            executor = HookExecutor()

            mock_result = MagicMock()
            mock_result.stdout = 'not json'

            with patch('subprocess.run', return_value=mock_result):
                result = executor._execute_single_hook(
                    {'type': 'command', 'command': 'echo test'},
                    'PreToolUse', 'Edit', {}
                )

            assert result['decision'] == 'allow'
            assert 'error' in result
            assert 'Invalid JSON' in result['error']

    def test_empty_stdout_returns_allow(self):
        """空の標準出力時はallowを返す"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager'):
            executor = HookExecutor()

            mock_result = MagicMock()
            mock_result.stdout = ''

            with patch('subprocess.run', return_value=mock_result):
                result = executor._execute_single_hook(
                    {'type': 'command', 'command': 'echo test'},
                    'PreToolUse', 'Edit', {}
                )

            assert result['decision'] == 'allow'

    def test_timeout_handling(self):
        """タイムアウト時はエラーを含む結果を返す"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager'):
            executor = HookExecutor()

            with patch('subprocess.run', side_effect=subprocess.TimeoutExpired('cmd', 60)):
                result = executor._execute_single_hook(
                    {'type': 'command', 'command': 'sleep 100', 'timeout': 60},
                    'PreToolUse', 'Edit', {}
                )

            assert result['decision'] == 'allow'
            assert 'error' in result
            assert 'timed out' in result['error']

    def test_exception_handling(self):
        """例外発生時はエラーを含む結果を返す"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager'):
            executor = HookExecutor()

            with patch('subprocess.run', side_effect=Exception('test error')):
                result = executor._execute_single_hook(
                    {'type': 'command', 'command': 'bad command'},
                    'PreToolUse', 'Edit', {}
                )

            assert result['decision'] == 'allow'
            assert 'error' in result
            assert 'test error' in result['error']


class TestExecuteBuiltinHook:
    """_execute_builtin_hook メソッドのテスト"""

    def test_unknown_hook_returns_none(self):
        """未知のフック名はNoneを返す"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager'):
            executor = HookExecutor()
            result = executor._execute_builtin_hook('unknown_hook', {})
            assert result is None

    def test_implementation_design_hook_execution(self):
        """implementation_designフックが実行される"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager'):
            executor = HookExecutor()

            mock_hook = MagicMock()
            mock_hook.is_session_processed.return_value = False
            mock_hook.should_process.return_value = True
            mock_hook.process.return_value = {'decision': 'allow', 'reason': 'ok'}

            executor.builtin_hooks['implementation_design'] = MagicMock(return_value=mock_hook)

            result = executor._execute_builtin_hook(
                'implementation_design', {'file_path': '/test.py'}
            )

            assert result['decision'] == 'allow'
            mock_hook.process.assert_called_once()

    def test_skip_processed_session(self):
        """処理済みセッションはスキップ"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager'):
            executor = HookExecutor()

            mock_hook = MagicMock()
            mock_hook.is_session_processed.return_value = True

            executor.builtin_hooks['implementation_design'] = MagicMock(return_value=mock_hook)

            result = executor._execute_builtin_hook(
                'implementation_design', {}, session_id='processed-session'
            )

            assert result is None

    def test_skip_non_matching_input(self):
        """should_processがFalseの場合はスキップ"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager'):
            executor = HookExecutor()

            mock_hook = MagicMock()
            mock_hook.is_session_processed.return_value = False
            mock_hook.should_process.return_value = False

            executor.builtin_hooks['implementation_design'] = MagicMock(return_value=mock_hook)

            result = executor._execute_builtin_hook(
                'implementation_design', {}
            )

            assert result is None

    def test_mark_session_processed_on_block(self):
        """blockの場合はセッションを処理済みにマーク"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager'):
            executor = HookExecutor()

            mock_hook = MagicMock()
            mock_hook.is_session_processed.return_value = False
            mock_hook.should_process.return_value = True
            mock_hook.process.return_value = {'decision': 'block', 'reason': 'test'}

            executor.builtin_hooks['implementation_design'] = MagicMock(return_value=mock_hook)

            executor._execute_builtin_hook(
                'implementation_design', {}, session_id='test-session'
            )

            mock_hook.mark_session_processed.assert_called_once_with('test-session')

    def test_exception_handling(self):
        """例外発生時はエラーを含む結果を返す"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager'):
            executor = HookExecutor()

            executor.builtin_hooks['implementation_design'] = MagicMock(
                side_effect=Exception('hook error')
            )

            result = executor._execute_builtin_hook('implementation_design', {})

            assert result['decision'] == 'allow'
            assert 'error' in result
            assert 'hook error' in result['error']


class TestExecuteHook:
    """execute_hook メソッドのテスト"""

    def test_no_hooks_returns_empty_list(self):
        """フックがない場合は空リストを返す"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager') as mock_manager_class:
            mock_manager = MagicMock()
            mock_manager.list_hooks.return_value = {}
            mock_manager_class.return_value = mock_manager

            executor = HookExecutor()
            results = executor.execute_hook('PreToolUse', 'Read', {})

            assert results == []

    def test_matching_hooks_are_executed(self):
        """マッチするフックが実行される"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager') as mock_manager_class:
            mock_manager = MagicMock()
            mock_manager.list_hooks.return_value = {
                'hooks': {
                    'PreToolUse': [
                        {
                            'matcher': 'Edit',
                            'hooks': [
                                {'type': 'command', 'command': 'echo test'}
                            ]
                        }
                    ]
                }
            }
            mock_manager_class.return_value = mock_manager

            executor = HookExecutor()

            mock_result = MagicMock()
            mock_result.stdout = json.dumps({'decision': 'allow'})

            with patch('subprocess.run', return_value=mock_result):
                results = executor.execute_hook('PreToolUse', 'Edit', {})

            assert len(results) == 1
            assert results[0]['decision'] == 'allow'

    def test_builtin_hook_for_edit_tool(self):
        """Editツールに対して内蔵フックが実行される"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager') as mock_manager_class:
            mock_manager = MagicMock()
            mock_manager.list_hooks.return_value = {}
            mock_manager_class.return_value = mock_manager

            executor = HookExecutor()

            mock_hook = MagicMock()
            mock_hook.is_session_processed.return_value = False
            mock_hook.should_process.return_value = True
            mock_hook.process.return_value = {'decision': 'allow'}

            executor.builtin_hooks['implementation_design'] = MagicMock(return_value=mock_hook)

            results = executor.execute_hook('PreToolUse', 'Edit', {'file_path': '/test.py'})

            assert len(results) == 1

    def test_builtin_hook_for_write_tool(self):
        """Writeツールに対して内蔵フックが実行される"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager') as mock_manager_class:
            mock_manager = MagicMock()
            mock_manager.list_hooks.return_value = {}
            mock_manager_class.return_value = mock_manager

            executor = HookExecutor()

            mock_hook = MagicMock()
            mock_hook.is_session_processed.return_value = False
            mock_hook.should_process.return_value = True
            mock_hook.process.return_value = {'decision': 'allow'}

            executor.builtin_hooks['implementation_design'] = MagicMock(return_value=mock_hook)

            results = executor.execute_hook('PreToolUse', 'Write', {'file_path': '/test.py'})

            assert len(results) == 1

    def test_builtin_hook_for_serena_tools(self):
        """mcp__serena__*ツールに対して内蔵フックが実行される"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager') as mock_manager_class:
            mock_manager = MagicMock()
            mock_manager.list_hooks.return_value = {}
            mock_manager_class.return_value = mock_manager

            executor = HookExecutor()

            mock_hook = MagicMock()
            mock_hook.is_session_processed.return_value = False
            mock_hook.should_process.return_value = True
            mock_hook.process.return_value = {'decision': 'allow'}

            executor.builtin_hooks['implementation_design'] = MagicMock(return_value=mock_hook)

            results = executor.execute_hook('PreToolUse', 'mcp__serena__create', {})

            assert len(results) == 1

    def test_no_builtin_hook_for_read_tool(self):
        """Readツールに対しては内蔵フックが実行されない"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager') as mock_manager_class:
            mock_manager = MagicMock()
            mock_manager.list_hooks.return_value = {}
            mock_manager_class.return_value = mock_manager

            executor = HookExecutor()
            results = executor.execute_hook('PreToolUse', 'Read', {})

            assert len(results) == 0


class TestRegisterHookViaCLI:
    """register_hook_via_cli メソッドのテスト"""

    def test_register_normal_command(self):
        """通常のコマンドを登録"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager') as mock_manager_class:
            mock_manager = MagicMock()
            mock_manager.add_hook.return_value = {'success': True}
            mock_manager_class.return_value = mock_manager

            executor = HookExecutor()
            result = executor.register_hook_via_cli(
                'PreToolUse', 'Edit', 'echo test', timeout=30, local=False
            )

            mock_manager.add_hook.assert_called_once_with(
                'PreToolUse', 'Edit', 'echo test', 30, False
            )
            assert result['success'] is True

    def test_register_python_script(self):
        """Pythonスクリプトを登録時はフルパスに変換"""
        with patch('src.infrastructure.hooks.hook_executor.HookManager') as mock_manager_class:
            mock_manager = MagicMock()
            mock_manager.add_hook.return_value = {'success': True}
            mock_manager_class.return_value = mock_manager

            executor = HookExecutor()
            result = executor.register_hook_via_cli(
                'PreToolUse', 'Edit', 'my_hook.py', local=True
            )

            # コマンドがpython3プレフィックス付きで呼ばれることを確認
            call_args = mock_manager.add_hook.call_args[0]
            assert 'python3' in call_args[2]
            assert 'my_hook.py' in call_args[2]



#!/usr/bin/env python3
"""フィクスチャキャプチャスクリプト

/tmp/claude/からテストフィクスチャを取り込む
"""

import json
import os
import shutil
from datetime import datetime
from pathlib import Path
from typing import Optional, List

# 相対インポート用
import sys
sys.path.insert(0, str(Path(__file__).parent))
from sanitizer import FixtureSanitizer


class FixtureCapture:
    """フィクスチャキャプチャ管理"""
    
    # キャプチャ元ディレクトリ
    SOURCE_DIR = Path("/tmp/claude")
    
    # フィクスチャ保存先
    FIXTURE_DIR = Path(__file__).parent.parent / "tests" / "fixtures" / "claude_code"
    
    # イベント種別の推定パターン
    EVENT_PATTERNS = {
        'pre_tool_use': ['PreToolUse', 'tool_input'],
        'post_tool_use': ['PostToolUse', 'tool_result'],
    }
    
    # ツール種別の推定パターン
    TOOL_PATTERNS = {
        'edit': ['Edit', 'file_path', 'old_string', 'new_string'],
        'write': ['Write', 'content'],
        'bash': ['Bash', 'command'],
        'read': ['Read'],
        'glob': ['Glob', 'pattern'],
        'grep': ['Grep'],
    }
    
    def __init__(self, source_dir: Path = None, fixture_dir: Path = None):
        """
        初期化
        
        Args:
            source_dir: キャプチャ元ディレクトリ
            fixture_dir: フィクスチャ保存先
        """
        self.source_dir = source_dir or self.SOURCE_DIR
        self.fixture_dir = fixture_dir or self.FIXTURE_DIR
        self.sanitizer = FixtureSanitizer()
    
    def detect_event_type(self, data: dict) -> str:
        """
        JSONデータからイベント種別を推定
        
        Args:
            data: パース済みJSONデータ
            
        Returns:
            イベント種別（pre_tool_use/post_tool_use/unknown）
        """
        json_str = json.dumps(data)
        
        for event_type, patterns in self.EVENT_PATTERNS.items():
            if any(pattern in json_str for pattern in patterns):
                return event_type
        
        return 'unknown'
    
    def detect_tool_type(self, data: dict) -> str:
        """
        JSONデータからツール種別を推定
        
        Args:
            data: パース済みJSONデータ
            
        Returns:
            ツール種別（edit/write/bash等/unknown）
        """
        # tool_nameフィールドから直接取得を試行
        tool_name = data.get('tool_name', '')
        if tool_name:
            return tool_name.lower()
        
        json_str = json.dumps(data)
        
        for tool_type, patterns in self.TOOL_PATTERNS.items():
            if any(pattern in json_str for pattern in patterns):
                return tool_type
        
        return 'unknown'
    
    def get_captured_files(self) -> List[Path]:
        """
        キャプチャ元ディレクトリのJSONファイル一覧を取得
        
        Returns:
            JSONファイルパスのリスト（更新日時順）
        """
        if not self.source_dir.exists():
            return []
        
        files = list(self.source_dir.glob("*.json"))
        # 更新日時でソート（新しい順）
        files.sort(key=lambda f: f.stat().st_mtime, reverse=True)
        return files
    
    def generate_fixture_name(self, data: dict, scenario: str = None) -> str:
        """
        フィクスチャファイル名を生成
        
        Args:
            data: パース済みJSONデータ
            scenario: シナリオ名（任意）
            
        Returns:
            ファイル名
        """
        event_type = self.detect_event_type(data)
        tool_type = self.detect_tool_type(data)
        date_str = datetime.now().strftime("%Y%m%d")
        
        if scenario:
            return f"{event_type}_{tool_type}_{scenario}_{date_str}.json"
        else:
            return f"{event_type}_{tool_type}_{date_str}.json"
    
    def capture_latest(self, 
                       count: int = 1, 
                       sanitize: bool = True,
                       scenario: str = None) -> List[Path]:
        """
        最新のキャプチャデータをフィクスチャとして保存
        
        Args:
            count: 取り込むファイル数
            sanitize: サニタイズを実行するか
            scenario: シナリオ名（ファイル名に含める）
            
        Returns:
            保存したファイルパスのリスト
        """
        captured_files = self.get_captured_files()
        if not captured_files:
            print("No captured files found in /tmp/claude/")
            return []
        
        saved_files = []
        
        for i, source_file in enumerate(captured_files[:count]):
            try:
                with open(source_file, 'r', encoding='utf-8') as f:
                    data = json.load(f)
                
                event_type = self.detect_event_type(data)
                tool_type = self.detect_tool_type(data)
                
                # 保存先ディレクトリ
                dest_dir = self.fixture_dir / event_type / tool_type
                dest_dir.mkdir(parents=True, exist_ok=True)
                
                # ファイル名生成
                fixture_name = self.generate_fixture_name(data, scenario)
                dest_path = dest_dir / fixture_name
                
                # 重複回避
                counter = 1
                while dest_path.exists():
                    base_name = fixture_name.replace('.json', f'_{counter}.json')
                    dest_path = dest_dir / base_name
                    counter += 1
                
                # サニタイズして保存
                if sanitize:
                    sanitized_data = self.sanitizer.sanitize_dict(data)
                    with open(dest_path, 'w', encoding='utf-8') as f:
                        json.dump(sanitized_data, f, ensure_ascii=False, indent=2)
                else:
                    shutil.copy2(source_file, dest_path)
                
                print(f"Captured: {source_file.name} -> {dest_path}")
                saved_files.append(dest_path)
                
            except Exception as e:
                print(f"Error processing {source_file}: {e}")
        
        return saved_files
    
    def list_fixtures(self) -> dict:
        """
        現在のフィクスチャ一覧を取得
        
        Returns:
            イベント・ツール別のフィクスチャ数
        """
        result = {}
        
        for event_dir in self.fixture_dir.iterdir():
            if not event_dir.is_dir() or event_dir.name.startswith('.'):
                continue
            
            result[event_dir.name] = {}
            
            for tool_dir in event_dir.iterdir():
                if not tool_dir.is_dir():
                    continue
                
                fixtures = list(tool_dir.glob("*.json"))
                result[event_dir.name][tool_dir.name] = len(fixtures)
        
        return result
    
    def validate_fixture(self, fixture_path: Path) -> dict:
        """
        フィクスチャの検証
        
        Args:
            fixture_path: フィクスチャファイルパス
            
        Returns:
            検証結果
        """
        result = {
            'valid': False,
            'errors': [],
            'warnings': []
        }
        
        try:
            with open(fixture_path, 'r', encoding='utf-8') as f:
                data = json.load(f)
            
            # 秘密情報チェック
            json_str = json.dumps(data)
            
            # ホームディレクトリパスが残っていないか
            if '/Users/' in json_str and '/Users/testuser' not in json_str:
                result['warnings'].append("Potential home directory path found")
            
            # セッションIDが残っていないか（サニタイズ後以外）
            import re
            uuid_pattern = r'[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}'
            uuids = re.findall(uuid_pattern, json_str)
            real_uuids = [u for u in uuids if u != '00000000-0000-0000-0000-000000000000']
            if real_uuids:
                result['warnings'].append(f"Potential session IDs found: {real_uuids[:3]}")
            
            result['valid'] = len(result['errors']) == 0
            
        except json.JSONDecodeError as e:
            result['errors'].append(f"Invalid JSON: {e}")
        except Exception as e:
            result['errors'].append(f"Error: {e}")
        
        return result


def main():
    """CLI エントリーポイント"""
    import argparse
    
    parser = argparse.ArgumentParser(description='Capture Claude Code fixtures')
    parser.add_argument('--count', '-n', type=int, default=1,
                       help='Number of files to capture')
    parser.add_argument('--sanitize', '-s', action='store_true', default=True,
                       help='Sanitize captured data')
    parser.add_argument('--no-sanitize', action='store_false', dest='sanitize',
                       help='Skip sanitization (dangerous!)')
    parser.add_argument('--scenario', help='Scenario name for fixture')
    parser.add_argument('--list', '-l', action='store_true',
                       help='List current fixtures')
    parser.add_argument('--validate', '-v', help='Validate a fixture file')
    
    args = parser.parse_args()
    
    capture = FixtureCapture()
    
    if args.list:
        fixtures = capture.list_fixtures()
        print("\nCurrent fixtures:")
        for event, tools in fixtures.items():
            print(f"\n  {event}/")
            for tool, count in tools.items():
                print(f"    {tool}/: {count} files")
        return
    
    if args.validate:
        result = capture.validate_fixture(Path(args.validate))
        print(f"\nValidation result: {'PASSED' if result['valid'] else 'FAILED'}")
        if result['errors']:
            print("Errors:")
            for e in result['errors']:
                print(f"  - {e}")
        if result['warnings']:
            print("Warnings:")
            for w in result['warnings']:
                print(f"  - {w}")
        return
    
    # デフォルト: キャプチャ実行
    saved = capture.capture_latest(
        count=args.count,
        sanitize=args.sanitize,
        scenario=args.scenario
    )
    
    if saved:
        print(f"\nCaptured {len(saved)} fixture(s)")
    else:
        print("\nNo files captured")


if __name__ == '__main__':
    main()



"""規約マッチング処理の基底クラス"""

from abc import ABC, abstractmethod
from typing import List


class BaseConventionMatcher(ABC):
    """規約マッチング処理の基底クラス"""
    
    @abstractmethod
    def matches_pattern(self, target: str, patterns: List[str]) -> bool:
        """
        対象がパターンにマッチするか確認
        
        Args:
            target: チェック対象の文字列
            patterns: パターンリスト
            
        Returns:
            マッチする場合True
        """
        pass


"""agent_spawn_guard.py のテスト

Agent tool経由のsub-agent直接起動ガードの動作を検証する。
- ビルトインホワイトリスト（Explore, Plan, statusline-setup, claude-code-guide）は許可
- team_name指定あり + promptパターン合致 → override注入(allow+updatedInput)
- team_name空白のみ → deny（strip()で空扱い）
- subagent（agent_context="subagent"）は制約対象外
- それ以外 → deny
"""

import json
import os
import subprocess
import sys
import tempfile

import pytest

# テスト対象スクリプトのパス
GUARD_SCRIPT = os.path.join(
    os.path.dirname(__file__),
    "..",
    ".claude",
    "plugins",
    "ticket-tasuki",
    "hooks",
    "agent_spawn_guard.py",
)
GUARD_SCRIPT = os.path.normpath(GUARD_SCRIPT)


def _run_guard(input_data: dict, env: dict | None = None) -> tuple[int, dict | None]:
    """ガードスクリプトを実行し、(終了コード, stdout JSONまたはNone) を返す"""
    proc = subprocess.run(
        [sys.executable, GUARD_SCRIPT],
        input=json.dumps(input_data),
        capture_output=True,
        text=True,
        timeout=10,
        env=env,
    )
    stdout_json = None
    if proc.stdout.strip():
        try:
            stdout_json = json.loads(proc.stdout)
        except json.JSONDecodeError:
            pass
    return proc.returncode, stdout_json


def _assert_override_output(out: dict, original_prompt: str) -> None:
    """override注入出力の構造を検証するヘルパー"""
    assert out is not None
    hook_output = out["hookSpecificOutput"]
    assert hook_output["permissionDecision"] == "allow"
    assert "updatedInput" in hook_output
    updated_prompt = hook_output["updatedInput"]["prompt"]
    assert updated_prompt.startswith(original_prompt + "\n\n")
    # override指示が付加されていることを確認
    assert len(updated_prompt) > len(original_prompt) + 2


def _make_agent_input(
    subagent_type: str = "",
    team_name: str = "",
    agent_context: str = "",
    prompt: str = "",
) -> dict:
    """PreToolUse Agent入力データを生成"""
    tool_input = {"subagent_type": subagent_type}
    if team_name:
        tool_input["team_name"] = team_name
    if prompt:
        tool_input["prompt"] = prompt
    data = {
        "hook_event_name": "PreToolUse",
        "tool_name": "Agent",
        "tool_input": tool_input,
    }
    if agent_context:
        data["agent_context"] = agent_context
    return data


class TestBuiltinWhitelist:
    """ビルトインホワイトリストの許可テスト"""

    @pytest.mark.parametrize("agent_type", ["Explore", "Plan"])
    def test_allow_builtin_types(self, agent_type):
        """ホワイトリスト対象はteam_name不要で許可"""
        data = _make_agent_input(subagent_type=agent_type, prompt="issue_1234 do something")
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is None  # 完全許可

    def test_allow_statusline_setup(self):
        """statusline-setupは許可（BUILTIN_WHITELISTに含まれる）"""
        data = _make_agent_input(subagent_type="statusline-setup", prompt="issue_1234")
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is None

    def test_allow_claude_code_guide(self):
        """claude-code-guideは許可（BUILTIN_WHITELISTに含まれる）"""
        data = _make_agent_input(subagent_type="claude-code-guide", prompt="issue_1234")
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is None

    def test_case_sensitive_whitelist(self):
        """ホワイトリストは大文字小文字を区別する"""
        data = _make_agent_input(subagent_type="explore")  # 小文字
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is not None
        assert out["hookSpecificOutput"]["permissionDecision"] == "deny"


class TestTeamNameHandling:
    """team_name指定による許可・拒否テスト"""

    def test_allow_with_team_name(self):
        """team_name指定あり+promptパターン合致→override注入(allow+updatedInput)"""
        data = _make_agent_input(
            subagent_type="general-purpose",
            team_name="my-team",
            prompt="issue_7579",
        )
        rc, out = _run_guard(data)

        assert rc == 0
        _assert_override_output(out, "issue_7579")

    def test_allow_with_team_name_no_config(self):
        """team_name指定あり＋config不在でもデフォルトoverride指示で許可"""
        data = _make_agent_input(
            subagent_type="general-purpose",
            team_name="any-team",
            prompt="issue_7947",
        )
        rc, out = _run_guard(data)

        assert rc == 0
        _assert_override_output(out, "issue_7947")

    @pytest.mark.parametrize("role", ["coder", "tech-lead", "tester", "pmo"])
    def test_allow_all_ticket_tasuki_roles_with_team_name(self, role):
        """team_name指定時に全ticket-tasukiロールがoverride注入付きで許可（promptパターン準拠）"""
        data = _make_agent_input(
            subagent_type=role,
            team_name="my-team",
            prompt="issue_7947",
        )
        rc, out = _run_guard(data)

        assert rc == 0
        _assert_override_output(out, "issue_7947")

    def test_block_without_team_name(self):
        """team_name未指定はdeny"""
        data = _make_agent_input(subagent_type="general-purpose")
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is not None
        assert out["hookSpecificOutput"]["permissionDecision"] == "deny"

    def test_whitespace_only_team_name_treated_as_empty(self):
        """team_name=" "（空白文字のみ）はstrip()で空扱い→deny"""
        data = _make_agent_input(subagent_type="general-purpose")
        # _make_agent_inputはteam_name truthyなのでtool_inputに含まれる
        data["tool_input"]["team_name"] = " "
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is not None
        assert out["hookSpecificOutput"]["permissionDecision"] == "deny"


class TestSubagentContext:
    """agent_context="subagent"の制約対象外テスト"""

    def test_subagent_context_bypasses_guard(self):
        """subagentからの呼び出しは制約対象外"""
        data = _make_agent_input(
            subagent_type="general-purpose",
            agent_context="subagent",
        )
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is None


class TestNonAgentTool:
    """Agent以外のtool_nameは対象外"""

    def test_ignore_non_agent_tool(self):
        """Agent以外のtool_nameはスキップ"""
        data = _make_agent_input(subagent_type="general-purpose")
        data["tool_name"] = "Task"
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is None


class TestIssueIdCheck:
    """issue_{id}トレーサビリティチェックのテスト"""

    def test_warn_when_issue_id_missing_builtin(self):
        """ビルトイン許可時にissue_id欠落でask警告"""
        data = _make_agent_input(subagent_type="Explore", prompt="do something")
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is not None
        assert out["hookSpecificOutput"]["permissionDecision"] == "ask"

    def test_deny_when_issue_id_missing_team_name(self):
        """team_name指定+非ビルトインでpromptパターン不一致→deny（promptパターン制限優先）"""
        data = _make_agent_input(
            subagent_type="general-purpose",
            team_name="my-team",
            prompt="no ticket reference here",
        )
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is not None
        assert out["hookSpecificOutput"]["permissionDecision"] == "deny"

    def test_no_warn_when_issue_id_present(self):
        """issue_id含む場合は警告なしで許可"""
        data = _make_agent_input(subagent_type="Explore", prompt="issue_1234 fix bug")
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is None

    def test_warn_when_prompt_empty(self):
        """prompt空文字でask警告"""
        data = _make_agent_input(subagent_type="Plan", prompt="")
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is not None
        assert out["hookSpecificOutput"]["permissionDecision"] == "ask"

    def test_deny_takes_priority_over_issue_id_warn(self):
        """deny判定はissue_id警告より優先"""
        data = _make_agent_input(subagent_type="general-purpose", prompt="no issue id")
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is not None
        assert out["hookSpecificOutput"]["permissionDecision"] == "deny"


class TestEdgeCases:
    """エッジケースのテスト"""

    def test_invalid_json_input(self):
        """不正なJSON入力は許可（exit 0）"""
        proc = subprocess.run(
            [sys.executable, GUARD_SCRIPT],
            input="invalid json",
            capture_output=True,
            text=True,
            timeout=10,
        )
        assert proc.returncode == 0
        assert proc.stdout.strip() == ""

    def test_empty_input(self):
        """空入力は許可"""
        proc = subprocess.run(
            [sys.executable, GUARD_SCRIPT],
            input="",
            capture_output=True,
            text=True,
            timeout=10,
        )
        assert proc.returncode == 0

    def test_empty_subagent_type_blocked(self):
        """subagent_type空文字はdeny"""
        data = _make_agent_input(subagent_type="")
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is not None
        assert out["hookSpecificOutput"]["permissionDecision"] == "deny"


class TestAdditionalScenarios:
    """追加受入テストシナリオ (issue_7947)"""

    def test_empty_string_team_name_deny(self):
        """team_name空文字はdeny"""
        data = _make_agent_input(subagent_type="general-purpose")
        data["tool_input"]["team_name"] = ""
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is not None
        assert out["hookSpecificOutput"]["permissionDecision"] == "deny"

    def test_tech_lead_without_team_name_deny(self):
        """tech-leadロール単独（team_nameなし）はdeny"""
        data = _make_agent_input(subagent_type="tech-lead")
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is not None
        assert out["hookSpecificOutput"]["permissionDecision"] == "deny"

    def test_tester_without_team_name_deny(self):
        """testerロール単独（team_nameなし）はdeny"""
        data = _make_agent_input(subagent_type="tester")
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is not None
        assert out["hookSpecificOutput"]["permissionDecision"] == "deny"

    def test_pmo_without_team_name_deny(self):
        """pmoロール単独（team_nameなし）はdeny"""
        data = _make_agent_input(subagent_type="pmo")
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is not None
        assert out["hookSpecificOutput"]["permissionDecision"] == "deny"

    def test_no_subagent_type_key_in_tool_input(self):
        """tool_inputにsubagent_typeキーなしはdeny"""
        data = {
            "hook_event_name": "PreToolUse",
            "tool_name": "Agent",
            "tool_input": {},
        }
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is not None
        assert out["hookSpecificOutput"]["permissionDecision"] == "deny"

    def test_subagent_context_without_team_name(self):
        """subagentコンテキスト+team_nameなしはバイパス（許可）"""
        data = _make_agent_input(
            subagent_type="general-purpose",
            agent_context="subagent",
        )
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is None

    def test_tab_only_team_name_deny(self):
        """team_nameタブ文字のみはstrip()で空扱い→deny"""
        data = _make_agent_input(subagent_type="general-purpose")
        data["tool_input"]["team_name"] = "\t"
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is not None
        assert out["hookSpecificOutput"]["permissionDecision"] == "deny"

    def test_team_name_with_surrounding_whitespace_allow(self):
        """team_name前後空白ありはstrip()後有効→override注入付き許可"""
        data = _make_agent_input(
            subagent_type="general-purpose",
            prompt="issue_7947",
        )
        data["tool_input"]["team_name"] = "  my-team  "
        rc, out = _run_guard(data)

        assert rc == 0
        _assert_override_output(out, "issue_7947")


class TestPromptPatternRestriction:
    """promptパターン制限のテスト（issue_8132）"""

    def test_valid_prompt_pattern_allowed(self):
        """issue_1234形式のpromptはoverride注入付き許可"""
        data = _make_agent_input(
            subagent_type="coder",
            team_name="my-team",
            prompt="issue_1234",
        )
        rc, out = _run_guard(data)

        assert rc == 0
        _assert_override_output(out, "issue_1234")

    def test_valid_prompt_pattern_6digits(self):
        """6桁のissue_idもoverride注入付き許可"""
        data = _make_agent_input(
            subagent_type="coder",
            team_name="my-team",
            prompt="issue_123456",
        )
        rc, out = _run_guard(data)

        assert rc == 0
        _assert_override_output(out, "issue_123456")

    def test_valid_prompt_pattern_1digit(self):
        """1桁のissue_idもoverride注入付き許可"""
        data = _make_agent_input(
            subagent_type="coder",
            team_name="my-team",
            prompt="issue_1",
        )
        rc, out = _run_guard(data)

        assert rc == 0
        _assert_override_output(out, "issue_1")

    def test_deny_prompt_with_extra_text(self):
        """issue_1234に追加テキストがある場合はdeny"""
        data = _make_agent_input(
            subagent_type="coder",
            team_name="my-team",
            prompt="issue_1234 implement feature",
        )
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is not None
        assert out["hookSpecificOutput"]["permissionDecision"] == "deny"

    def test_deny_prompt_freeform(self):
        """自由文promptはdeny"""
        data = _make_agent_input(
            subagent_type="coder",
            team_name="my-team",
            prompt="implement the login feature",
        )
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is not None
        assert out["hookSpecificOutput"]["permissionDecision"] == "deny"

    def test_deny_prompt_empty(self):
        """空promptはdeny"""
        data = _make_agent_input(
            subagent_type="coder",
            team_name="my-team",
        )
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is not None
        assert out["hookSpecificOutput"]["permissionDecision"] == "deny"

    def test_deny_prompt_7digits(self):
        """7桁のissue_idはdeny（上限6桁）"""
        data = _make_agent_input(
            subagent_type="coder",
            team_name="my-team",
            prompt="issue_1234567",
        )
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is not None
        assert out["hookSpecificOutput"]["permissionDecision"] == "deny"

    def test_builtin_exempt_from_prompt_pattern(self):
        """ビルトインsubagent_typeはpromptパターン制限対象外"""
        data = _make_agent_input(
            subagent_type="Explore",
            team_name="my-team",
            prompt="explore the codebase for auth patterns",
        )
        rc, out = _run_guard(data)

        assert rc == 0
        # ビルトインはprompt自由文OK（issue_id欠落のask警告のみ）
        if out is not None:
            assert out["hookSpecificOutput"]["permissionDecision"] == "ask"

    def test_builtin_plan_exempt_from_prompt_pattern(self):
        """Plan subagent_typeもpromptパターン制限対象外"""
        data = _make_agent_input(
            subagent_type="Plan",
            team_name="my-team",
            prompt="issue_1234 plan the implementation",
        )
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is None

    @pytest.mark.parametrize("role", ["coder", "tech-lead", "tester", "pmo", "researcher"])
    def test_all_roles_require_prompt_pattern(self, role):
        """全ticket-tasukiロールにpromptパターン制限が適用される"""
        data = _make_agent_input(
            subagent_type=role,
            team_name="my-team",
            prompt="issue_8132 do work",
        )
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is not None
        assert out["hookSpecificOutput"]["permissionDecision"] == "deny"

    @pytest.mark.parametrize("role", ["coder", "tech-lead", "tester", "pmo", "researcher"])
    def test_all_roles_allow_valid_prompt(self, role):
        """全ロールで正しいpromptパターンならoverride注入付き許可"""
        data = _make_agent_input(
            subagent_type=role,
            team_name="my-team",
            prompt="issue_8132",
        )
        rc, out = _run_guard(data)

        assert rc == 0
        _assert_override_output(out, "issue_8132")


class TestOverrideInjection:
    """override注入（updatedInput）のテスト（issue_8133）"""

    def test_override_output_structure(self):
        """override出力がupdatedInput構造を持つことを検証"""
        data = _make_agent_input(
            subagent_type="coder",
            team_name="my-team",
            prompt="issue_1234",
        )
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is not None
        hook_out = out["hookSpecificOutput"]
        assert hook_out["hookEventName"] == "PreToolUse"
        assert hook_out["permissionDecision"] == "allow"
        assert "updatedInput" in hook_out
        assert "prompt" in hook_out["updatedInput"]

    def test_override_prompt_contains_original(self):
        """注入後のpromptが元のpromptを含むことを検証"""
        data = _make_agent_input(
            subagent_type="coder",
            team_name="my-team",
            prompt="issue_5678",
        )
        rc, out = _run_guard(data)

        assert rc == 0
        updated_prompt = out["hookSpecificOutput"]["updatedInput"]["prompt"]
        assert updated_prompt.startswith("issue_5678\n\n")

    def test_override_prompt_contains_instruction(self):
        """注入後のpromptにRedmine読み込み指示が含まれることを検証"""
        data = _make_agent_input(
            subagent_type="coder",
            team_name="my-team",
            prompt="issue_9999",
        )
        rc, out = _run_guard(data)

        assert rc == 0
        updated_prompt = out["hookSpecificOutput"]["updatedInput"]["prompt"]
        # デフォルトまたはconfig設定のoverride指示が含まれること
        assert "Redmine" in updated_prompt or "get_issue_detail_tool" in updated_prompt

    def test_builtin_no_override(self):
        """ビルトインsubagent_typeにはoverride注入されない"""
        data = _make_agent_input(
            subagent_type="Explore",
            team_name="my-team",
            prompt="issue_1234 explore something",
        )
        rc, out = _run_guard(data)

        assert rc == 0
        # ビルトインは直接許可（updatedInputなし）
        assert out is None

    def test_override_not_applied_on_deny(self):
        """deny時にはoverride注入されない"""
        data = _make_agent_input(
            subagent_type="coder",
            team_name="my-team",
            prompt="free text prompt",
        )
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is not None
        assert out["hookSpecificOutput"]["permissionDecision"] == "deny"
        assert "updatedInput" not in out["hookSpecificOutput"]


class TestConfigYamlMessages:
    """config.yamlからのブロックメッセージ読み込みテスト（issue_8137）"""

    def test_block_message_from_config(self):
        """config.yamlのblock_message_templateが使用される"""
        data = _make_agent_input(subagent_type="coder", prompt="issue_1234")
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is not None
        reason = out["hookSpecificOutput"]["permissionDecisionReason"]
        assert "sub-agent直接起動の制限" in reason
        assert 'subagent_type="coder"' in reason

    def test_prompt_pattern_block_message_from_config(self):
        """config.yamlのprompt_pattern_block_messageが使用される"""
        data = _make_agent_input(
            subagent_type="coder",
            team_name="my-team",
            prompt="invalid prompt",
        )
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is not None
        reason = out["hookSpecificOutput"]["permissionDecisionReason"]
        assert "promptパターン制限" in reason
        assert '"invalid prompt"' in reason

    def test_issue_id_warn_message_from_config(self):
        """config.yamlのissue_id_warn_messageが使用される"""
        data = _make_agent_input(subagent_type="Explore", prompt="no issue id")
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is not None
        reason = out["hookSpecificOutput"]["permissionDecisionReason"]
        assert "issue_{id}" in reason

    def test_fallback_default_messages(self):
        """デフォルトメッセージの内容が正しく出力される"""
        data = _make_agent_input(subagent_type="unknown-role", prompt="issue_999")
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is not None
        reason = out["hookSpecificOutput"]["permissionDecisionReason"]
        assert "sub-agent直接起動の制限" in reason


class TestSubagentRedmineFlowConfig:
    """検証項目(d): subagent規約にget_issue_detail_toolフローが含まれること（issue_8139）

    config.yamlの各subagent_type override設定に
    get_issue_detail_tool規約文言が含まれることを検証する。
    """

    @pytest.fixture(autouse=True)
    def _load_config(self):
        """config.yamlのsession_startup.overridesセクションを読み込む"""
        import yaml

        # プロジェクトルートのconfig.yamlを使用
        config_path = os.path.join(
            os.path.dirname(__file__), "..", ".claude-nagger", "config.yaml"
        )
        config_path = os.path.normpath(config_path)
        with open(config_path, encoding="utf-8") as f:
            data = yaml.safe_load(f)
        self.overrides = data.get("session_startup", {}).get("overrides", {})
        self.subagent_types = self.overrides.get("subagent_types", {})

    @pytest.mark.parametrize("role", ["coder", "tester", "tech-lead", "pmo"])
    def test_role_contains_get_issue_detail_tool(self, role):
        """各ロールのoverride設定にget_issue_detail_tool規約文言が含まれる"""
        role_config = self.subagent_types.get(role, {})
        main_text = (
            role_config.get("messages", {})
            .get("first_time", {})
            .get("main_text", "")
        )
        assert "get_issue_detail_tool" in main_text, (
            f"{role}のoverride設定にget_issue_detail_tool文言が含まれていない"
        )

    def test_researcher_has_get_issue_detail_tool(self):
        """researcherのoverride設定にget_issue_detail_tool規約文言が含まれる

        researcherはsubagent_defaultフォールバックの可能性があるため個別検証。
        """
        researcher_config = self.subagent_types.get("researcher", {})
        main_text = (
            researcher_config.get("messages", {})
            .get("first_time", {})
            .get("main_text", "")
        )
        assert "get_issue_detail_tool" in main_text, (
            "researcherのoverride設定にget_issue_detail_tool文言が含まれていない。"
            "subagent_defaultフォールバックではget_issue_detail_tool規約が欠落する"
        )

    def test_override_instruction_contains_get_issue_detail_tool(self):
        """agent_spawn_guardのoverride_instructionにget_issue_detail_tool文言が含まれる"""
        import yaml

        config_path = os.path.join(
            os.path.dirname(__file__), "..", ".claude-nagger", "config.yaml"
        )
        config_path = os.path.normpath(config_path)
        with open(config_path, encoding="utf-8") as f:
            data = yaml.safe_load(f)
        guard_config = data.get("agent_spawn_guard", {})
        instruction = guard_config.get("override_instruction", "")
        assert "get_issue_detail_tool" in instruction, (
            "override_instructionにget_issue_detail_tool文言が含まれていない"
        )


class TestConfigYamlMessageOverride:
    """検証項目(e): config.yamlメッセージテンプレート書き換えの反映確認（issue_8139）

    config.yamlのメッセージテンプレートを書き換えた状態でguardスクリプトを実行し、
    変更がdeny/warn時のメッセージに正しく反映されることを検証する。
    """

    @pytest.fixture
    def custom_config_dir(self, tmp_path):
        """カスタムメッセージを持つconfig.yamlを含む一時ディレクトリを作成"""
        import shutil
        import yaml

        # guardスクリプトの親ディレクトリ構造を再現
        # agent_spawn_guard.py は script_dir/../.claude-nagger/config.yaml を参照
        hooks_dir = tmp_path / "hooks"
        hooks_dir.mkdir()
        config_dir = tmp_path / ".claude-nagger"
        config_dir.mkdir()

        # カスタムメッセージのconfig.yaml
        custom_config = {
            "agent_spawn_guard": {
                "block_message_template": 'カスタムブロック: type="{subagent_type}"',
                "prompt_pattern_block_message": 'カスタムパターン拒否: prompt="{prompt}"',
                "issue_id_warn_message": "カスタム警告: issue_idが必要です",
                "override_instruction": "カスタムoverride指示文",
            }
        }
        config_file = config_dir / "config.yaml"
        with open(config_file, "w", encoding="utf-8") as f:
            yaml.dump(custom_config, f, allow_unicode=True)

        # guardスクリプトをコピー
        shutil.copy2(GUARD_SCRIPT, hooks_dir / "agent_spawn_guard.py")

        return hooks_dir / "agent_spawn_guard.py"

    def _run_custom_guard(self, script_path, input_data):
        """カスタムconfig環境でguardスクリプトを実行"""
        proc = subprocess.run(
            [sys.executable, str(script_path)],
            input=json.dumps(input_data),
            capture_output=True,
            text=True,
            timeout=10,
        )
        stdout_json = None
        if proc.stdout.strip():
            try:
                stdout_json = json.loads(proc.stdout)
            except json.JSONDecodeError:
                pass
        return proc.returncode, stdout_json

    def test_custom_block_message_reflected(self, custom_config_dir):
        """書き換えたblock_message_templateがdeny時に反映される"""
        data = _make_agent_input(subagent_type="coder", prompt="issue_1234")
        rc, out = self._run_custom_guard(custom_config_dir, data)

        assert rc == 0
        assert out is not None
        reason = out["hookSpecificOutput"]["permissionDecisionReason"]
        assert "カスタムブロック" in reason
        assert 'type="coder"' in reason

    def test_custom_prompt_pattern_block_reflected(self, custom_config_dir):
        """書き換えたprompt_pattern_block_messageがpromptパターン拒否時に反映される"""
        data = _make_agent_input(
            subagent_type="coder",
            team_name="my-team",
            prompt="invalid prompt",
        )
        rc, out = self._run_custom_guard(custom_config_dir, data)

        assert rc == 0
        assert out is not None
        reason = out["hookSpecificOutput"]["permissionDecisionReason"]
        assert "カスタムパターン拒否" in reason
        assert '"invalid prompt"' in reason

    def test_custom_issue_id_warn_reflected(self, custom_config_dir):
        """書き換えたissue_id_warn_messageが警告時に反映される"""
        data = _make_agent_input(subagent_type="Explore", prompt="no issue id")
        rc, out = self._run_custom_guard(custom_config_dir, data)

        assert rc == 0
        assert out is not None
        reason = out["hookSpecificOutput"]["permissionDecisionReason"]
        assert "カスタム警告" in reason

    def test_custom_override_instruction_in_updated_input(self, custom_config_dir):
        """書き換えたoverride_instructionがupdatedInputに反映される"""
        data = _make_agent_input(
            subagent_type="coder",
            team_name="my-team",
            prompt="issue_1234",
        )
        rc, out = self._run_custom_guard(custom_config_dir, data)

        assert rc == 0
        assert out is not None
        updated_prompt = out["hookSpecificOutput"]["updatedInput"]["prompt"]
        assert "カスタムoverride指示文" in updated_prompt
        assert "issue_1234" in updated_prompt


class TestConfigYamlContentValidation:
    """config.yaml内容の妥当性検証テスト（issue_8139チューニング）

    config.yaml変更後の内容が期待通りであることを直接検証する。
    """

    @pytest.fixture(autouse=True)
    def _load_config(self):
        """config.yamlを読み込む"""
        import yaml

        config_path = os.path.join(
            os.path.dirname(__file__), "..", ".claude-nagger", "config.yaml"
        )
        config_path = os.path.normpath(config_path)
        with open(config_path, encoding="utf-8") as f:
            self.config = yaml.safe_load(f)

    def test_prompt_pattern_block_message_contains_reason(self):
        """prompt_pattern_block_messageに理由行が含まれる"""
        guard = self.config.get("agent_spawn_guard", {})
        msg = guard.get("prompt_pattern_block_message", "")
        assert "理由:" in msg, "prompt_pattern_block_messageに理由行が含まれていない"
        assert "Redmine連携" in msg

    def test_override_instruction_contains_ticket_reference(self):
        """override_instructionにチケット番号参照の指示が含まれる"""
        guard = self.config.get("agent_spawn_guard", {})
        instruction = guard.get("override_instruction", "")
        assert "promptに指定されたチケット番号" in instruction

    def test_override_instruction_in_updated_input(self):
        """override_instructionが実際のupdatedInputに反映される"""
        data = _make_agent_input(
            subagent_type="coder",
            team_name="my-team",
            prompt="issue_1234",
        )
        rc, out = _run_guard(data)

        assert rc == 0
        assert out is not None
        updated_prompt = out["hookSpecificOutput"]["updatedInput"]["prompt"]
        assert "promptに指定されたチケット番号" in updated_prompt



"""データベースインフラストラクチャ"""

from domain.models.records import HookLogRecord, SessionRecord, SubagentRecord
from infrastructure.db.convention_log_repository import ConventionLogRepository
from infrastructure.db.hook_log_repository import HookLogRepository
from infrastructure.db.nagger_state_db import NaggerStateDB
from infrastructure.db.session_repository import SessionRepository
from infrastructure.db.subagent_history_repository import SubagentHistoryRepository
from infrastructure.db.subagent_repository import SubagentRepository, SUBAGENT_TOOL_NAMES

__all__ = [
    "NaggerStateDB",
    "SubagentRecord",
    "SessionRecord",
    "HookLogRecord",
    "ConventionLogRepository",
    "HookLogRepository",
    "SessionRepository",
    "SubagentHistoryRepository",
    "SubagentRepository",
    "SUBAGENT_TOOL_NAMES",
]



"""FileConventionMatcherのテスト"""

import pytest
from pathlib import Path
import tempfile
import yaml
from src.domain.services.file_convention_matcher import FileConventionMatcher, ConventionRule


class TestFileConventionMatcher:
    """FileConventionMatcherのテストクラス"""

    @pytest.fixture
    def temp_rules_file(self):
        """テスト用の一時ルールファイルを作成"""
        with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml', delete=False) as f:
            rules_data = {
                'rules': [
                    {
                        'name': 'Test Rule 1',
                        'patterns': ['**/test.pu', 'test/*.pu'],
                        'convention_doc': '@test/doc1.md',
                        'severity': 'block',
                        'message': 'Test message 1'
                    },
                    {
                        'name': 'Test Rule 2',
                        'patterns': ['app/**/*.rb'],
                        'convention_doc': '@test/doc2.md',
                        'severity': 'warn',
                        'message': 'Test message 2'
                    }
                ]
            }
            yaml.dump(rules_data, f)
            temp_path = Path(f.name)
        
        yield temp_path
        
        # クリーンアップ
        temp_path.unlink()

    def test_load_rules(self, temp_rules_file):
        """ルールファイルの読み込みテスト"""
        matcher = FileConventionMatcher(temp_rules_file)
        
        assert len(matcher.rules) == 2
        assert matcher.rules[0].name == 'Test Rule 1'
        assert matcher.rules[0].severity == 'block'
        assert matcher.rules[1].name == 'Test Rule 2'
        assert matcher.rules[1].severity == 'warn'

    def test_matches_pattern_exact(self, temp_rules_file):
        """完全一致パターンのテスト"""
        matcher = FileConventionMatcher(temp_rules_file)
        
        # test/*.puパターンのテスト
        assert matcher.matches_pattern('test/file.pu', ['test/*.pu'])
        assert not matcher.matches_pattern('other/file.pu', ['test/*.pu'])

    def test_matches_pattern_wildcard(self, temp_rules_file):
        """ワイルドカードパターンのテスト"""
        matcher = FileConventionMatcher(temp_rules_file)
        
        # **パターンのテスト
        assert matcher.matches_pattern('deep/nested/test.pu', ['**/test.pu'])
        assert matcher.matches_pattern('test.pu', ['**/test.pu'])
        assert not matcher.matches_pattern('deep/nested/other.pu', ['**/test.pu'])
        
        # app/**/*.rbパターンのテスト
        assert matcher.matches_pattern('app/models/user.rb', ['app/**/*.rb'])
        assert matcher.matches_pattern('app/controllers/api/v1/users.rb', ['app/**/*.rb'])
        assert not matcher.matches_pattern('lib/tasks/user.rb', ['app/**/*.rb'])

    def test_check_file(self, temp_rules_file):
        """ファイルチェック機能のテスト"""
        matcher = FileConventionMatcher(temp_rules_file)
        
        # マッチするファイル
        rules = matcher.check_file('some/path/test.pu')
        assert len(rules) == 1
        assert rules[0].name == 'Test Rule 1'
        assert rules[0].severity == 'block'
        
        # マッチしないファイル
        rules = matcher.check_file('other/file.txt')
        assert rules == []

    def test_get_confirmation_message(self, temp_rules_file):
        """確認メッセージ生成のテスト"""
        matcher = FileConventionMatcher(temp_rules_file)

        # マッチするファイルの場合
        results = matcher.get_confirmation_message('test/example.pu')
        assert len(results) == 1
        result = results[0]
        assert result['rule_name'] == 'Test Rule 1'
        assert result['severity'] == 'block'
        # メッセージフォーマット: "⚠️  {rule.message}" (convention_docは含まない)
        assert 'Test message 1' in result['message']

        # マッチしないファイルの場合
        results = matcher.get_confirmation_message('other.txt')
        assert results == []

    def test_list_rules(self, temp_rules_file):
        """ルール一覧取得のテスト"""
        matcher = FileConventionMatcher(temp_rules_file)
        
        rules = matcher.list_rules()
        assert len(rules) == 2
        assert rules[0]['name'] == 'Test Rule 1'
        assert rules[0]['patterns'] == ['**/test.pu', 'test/*.pu']
        assert rules[1]['name'] == 'Test Rule 2'

    def test_reload_rules(self, temp_rules_file):
        """ルールリロードのテスト"""
        matcher = FileConventionMatcher(temp_rules_file)
        initial_rules = len(matcher.rules)
        
        # ファイルを更新
        with open(temp_rules_file, 'w') as f:
            new_data = {
                'rules': [
                    {
                        'name': 'New Rule',
                        'patterns': ['new/*.txt'],
                        'convention_doc': '@new/doc.md',
                        'severity': 'warn',
                        'message': 'New message'
                    }
                ]
            }
            yaml.dump(new_data, f)
        
        # リロード
        matcher.reload_rules()
        
        assert len(matcher.rules) == 1
        assert matcher.rules[0].name == 'New Rule'

    def test_empty_rules_file(self):
        """空のルールファイルのテスト"""
        with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml', delete=False) as f:
            yaml.dump({}, f)
            temp_path = Path(f.name)
        
        try:
            matcher = FileConventionMatcher(temp_path)
            assert len(matcher.rules) == 0
            assert matcher.check_file('any/file.txt') == []
        finally:
            temp_path.unlink()

    def test_nonexistent_rules_file(self):
        """存在しないルールファイルのテスト"""
        matcher = FileConventionMatcher(Path('/nonexistent/file.yaml'))
        assert len(matcher.rules) == 0
        assert matcher.check_file('any/file.txt') == []

    def test_check_file_multiple_rules_match(self):
        """複数ルールにマッチするケース"""
        with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml', delete=False) as f:
            rules_data = {
                'rules': [
                    {
                        'name': 'PUファイル規約',
                        'patterns': ['**/*.pu'],
                        'severity': 'block',
                        'message': 'PUファイル変更確認'
                    },
                    {
                        'name': 'テストディレクトリ規約',
                        'patterns': ['test/**/*'],
                        'severity': 'warn',
                        'message': 'テストファイル変更注意'
                    },
                    {
                        'name': '無関係規約',
                        'patterns': ['app/**/*.rb'],
                        'severity': 'warn',
                        'message': 'Rubyファイル注意'
                    }
                ]
            }
            yaml.dump(rules_data, f)
            temp_path = Path(f.name)

        try:
            matcher = FileConventionMatcher(temp_path)
            rules = matcher.check_file('test/example.pu')
            # PUファイル規約とテストディレクトリ規約の2つにマッチ
            assert len(rules) == 2
            assert rules[0].name == 'PUファイル規約'
            assert rules[1].name == 'テストディレクトリ規約'
        finally:
            temp_path.unlink()


class TestGlobstarPatterns:
    """GLOBSTARパターン(**/)の詳細テスト"""

    @pytest.fixture
    def matcher(self):
        """テスト用マッチャーを作成"""
        with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml', delete=False) as f:
            yaml.dump({'rules': []}, f)
            temp_path = Path(f.name)
        m = FileConventionMatcher(temp_path)
        temp_path.unlink()
        return m

    def test_double_globstar_directory_all_files(self, matcher):
        """**/test/** - どこにあるtest/でもその中の全ファイル"""
        pattern = ["**/test/**"]
        # test直下
        assert matcher.matches_pattern('test/file.py', pattern)
        # testサブディレクトリ
        assert matcher.matches_pattern('test/subdir/file.py', pattern)
        assert matcher.matches_pattern('test/subdir/deep/file.py', pattern)
        # src/test以下
        assert matcher.matches_pattern('src/test/file.py', pattern)
        assert matcher.matches_pattern('src/test/utils/helper.py', pattern)
        # マッチしないケース
        assert not matcher.matches_pattern('tests/file.py', pattern)  # testsはtestと異なる
        assert not matcher.matches_pattern('src/testing/file.py', pattern)

    def test_globstar_directory_specific_extension(self, matcher):
        """**/apps/**/*.scss - どこにあるapps/でも全階層の.scssファイル"""
        pattern = ["**/apps/**/*.scss"]
        # ルート直下apps
        assert matcher.matches_pattern('apps/style.scss', pattern)
        assert matcher.matches_pattern('apps/components/button.scss', pattern)
        # src/apps
        assert matcher.matches_pattern('src/apps/theme.scss', pattern)
        assert matcher.matches_pattern('packages/ui/apps/styles/main.scss', pattern)
        # 拡張子が異なる
        assert not matcher.matches_pattern('apps/style.css', pattern)
        assert not matcher.matches_pattern('apps/style.sass', pattern)

    def test_single_level_only(self, matcher):
        """src/*.ts - src直下のみ（サブディレクトリ除外）"""
        pattern = ["src/*.ts"]
        # 直下のみマッチ
        assert matcher.matches_pattern('src/index.ts', pattern)
        assert matcher.matches_pattern('src/utils.ts', pattern)
        # サブディレクトリはマッチしない
        assert not matcher.matches_pattern('src/lib/helper.ts', pattern)
        assert not matcher.matches_pattern('src/components/Button.ts', pattern)

    def test_globstar_with_prefix_pattern(self, matcher):
        """**/test_*.py - 全階層のtest_で始まるpyファイル"""
        pattern = ["**/test_*.py"]
        assert matcher.matches_pattern('test_main.py', pattern)
        assert matcher.matches_pattern('tests/test_utils.py', pattern)
        assert matcher.matches_pattern('src/tests/unit/test_handler.py', pattern)
        # マッチしない
        assert not matcher.matches_pattern('tests/utils.py', pattern)
        assert not matcher.matches_pattern('test_main.js', pattern)

    def test_globstar_with_suffix_pattern(self, matcher):
        """**/*_test.go - 全階層の_testで終わるgoファイル"""
        pattern = ["**/*_test.go"]
        assert matcher.matches_pattern('main_test.go', pattern)
        assert matcher.matches_pattern('pkg/handler_test.go', pattern)
        assert matcher.matches_pattern('internal/service/user_test.go', pattern)
        # マッチしない
        assert not matcher.matches_pattern('pkg/handler.go', pattern)
        assert not matcher.matches_pattern('main_test.rs', pattern)

    def test_root_directory_globstar(self, matcher):
        """apps/**/*.tsx - ルート直下apps以下の全階層"""
        pattern = ["apps/**/*.tsx"]
        # ルート直下のapps
        assert matcher.matches_pattern('apps/App.tsx', pattern)
        assert matcher.matches_pattern('apps/components/Button.tsx', pattern)
        assert matcher.matches_pattern('apps/pages/home/index.tsx', pattern)
        # src/appsはマッチしない（ルート直下のappsのみ）
        assert not matcher.matches_pattern('src/apps/App.tsx', pattern)

    def test_all_files_with_extension(self, matcher):
        """**/*.scss - 全階層の.scssファイル"""
        pattern = ["**/*.scss"]
        assert matcher.matches_pattern('style.scss', pattern)
        assert matcher.matches_pattern('src/style.scss', pattern)
        assert matcher.matches_pattern('src/components/button/style.scss', pattern)
        # 拡張子が異なる
        assert not matcher.matches_pattern('style.css', pattern)


class TestAbsolutePathConversion:
    """絶対パス→相対パス変換のテスト（#4074対応）"""

    @pytest.fixture
    def matcher(self):
        """テスト用マッチャーを作成"""
        with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml', delete=False) as f:
            yaml.dump({'rules': []}, f)
            temp_path = Path(f.name)
        m = FileConventionMatcher(temp_path)
        temp_path.unlink()
        return m

    def test_absolute_path_under_cwd(self, matcher, monkeypatch):
        """CWD配下の絶対パスが相対パスに変換されてマッチする"""
        # CWDを/myapp/Source/railsに設定
        monkeypatch.chdir('/tmp')
        
        # /tmp配下のパスをテスト
        pattern = ["app/views/**/*.erb"]
        
        # 一時的にディレクトリ構造を作成
        test_dir = Path('/tmp/app/views/shared')
        test_dir.mkdir(parents=True, exist_ok=True)
        
        try:
            # 絶対パスが相対パスに変換されてマッチする
            assert matcher.matches_pattern('/tmp/app/views/shared/test.erb', pattern)
            # 相対パスも引き続きマッチする
            assert matcher.matches_pattern('app/views/shared/test.erb', pattern)
        finally:
            # クリーンアップ
            import shutil
            shutil.rmtree('/tmp/app', ignore_errors=True)

    def test_absolute_path_not_under_cwd(self, matcher, monkeypatch):
        """CWD外の絶対パスはそのまま使用される（マッチしない）"""
        monkeypatch.chdir('/tmp')
        pattern = ["app/**/*.rb"]
        
        # /other/pathはCWD(/tmp)の配下ではないのでマッチしない
        assert not matcher.matches_pattern('/other/path/app/models/user.rb', pattern)

    def test_relative_path_unchanged(self, matcher):
        """相対パスはそのまま処理される"""
        pattern = ["src/**/*.py"]
        
        assert matcher.matches_pattern('src/main.py', pattern)
        assert matcher.matches_pattern('src/lib/utils.py', pattern)
        assert not matcher.matches_pattern('other/main.py', pattern)


class TestErrorHandling:
    """エラーハンドリングのテスト"""

    def test_load_rules_yaml_error(self):
        """YAML構文エラー"""
        with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml', delete=False) as f:
            f.write("invalid: yaml: content:\n  - broken")
            temp_path = Path(f.name)

        try:
            matcher = FileConventionMatcher(rules_file=temp_path)
            assert len(matcher.rules) == 0
        finally:
            temp_path.unlink()

    def test_load_rules_missing_required_field(self):
        """必須フィールド欠落（KeyError）"""
        with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml', delete=False) as f:
            rules_data = {
                'rules': [
                    {
                        'name': 'Missing patterns',
                        'message': 'No patterns field'
                    }
                ]
            }
            yaml.dump(rules_data, f)
            temp_path = Path(f.name)

        try:
            matcher = FileConventionMatcher(rules_file=temp_path)
            assert len(matcher.rules) == 0
        finally:
            temp_path.unlink()

    def test_invalid_pattern_skipped(self):
        """無効なパターンがスキップされる"""
        with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml', delete=False) as f:
            rules_data = {
                'rules': [
                    {
                        'name': 'Rule with valid pattern',
                        'patterns': ['valid/*.py'],
                        'severity': 'warn',
                        'message': 'Test message'
                    }
                ]
            }
            yaml.dump(rules_data, f)
            temp_path = Path(f.name)

        try:
            matcher = FileConventionMatcher(rules_file=temp_path)
            # 無効なパターン（Noneなど）を直接テスト
            # matches_patternは例外をキャッチしてFalseを返す
            result = matcher.matches_pattern('test.py', [None])
            assert result is False
        finally:
            temp_path.unlink()


class TestExcludePatterns:
    """exclude_patterns（除外パターン）のテスト"""

    @pytest.fixture
    def matcher(self):
        """テスト用マッチャーを作成"""
        with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml', delete=False) as f:
            yaml.dump({'rules': []}, f)
            temp_path = Path(f.name)
        m = FileConventionMatcher(temp_path)
        temp_path.unlink()
        return m

    def test_exclude_pattern_prevents_match(self, matcher):
        """除外パターンにマッチする場合、ルールが非適用になる"""
        patterns = ["**/*"]
        exclude_patterns = ["vibes/docs/**"]

        # 除外パターンにマッチ → False
        assert not matcher.matches_pattern('vibes/docs/rules/test.md', patterns, exclude_patterns)
        assert not matcher.matches_pattern('vibes/docs/README.md', patterns, exclude_patterns)
        assert not matcher.matches_pattern('vibes/docs/specs/arch.md', patterns, exclude_patterns)

    def test_exclude_pattern_non_match_allows_rule(self, matcher):
        """除外パターンにマッチしない場合、ルールが適用される"""
        patterns = ["**/*"]
        exclude_patterns = ["vibes/docs/**"]

        # 除外パターンにマッチしない → True（ルール適用）
        assert matcher.matches_pattern('src/main.py', patterns, exclude_patterns)
        assert matcher.matches_pattern('tests/test_main.py', patterns, exclude_patterns)
        assert matcher.matches_pattern('README.md', patterns, exclude_patterns)

    def test_exclude_patterns_empty_preserves_behavior(self, matcher):
        """exclude_patternsが空リストの場合、既存動作を維持"""
        patterns = ["**/*"]

        assert matcher.matches_pattern('any/file.txt', patterns, [])
        assert matcher.matches_pattern('src/main.py', patterns, [])

    def test_exclude_patterns_none_preserves_behavior(self, matcher):
        """exclude_patternsがNoneの場合、既存動作を維持"""
        patterns = ["**/*"]

        assert matcher.matches_pattern('any/file.txt', patterns, None)
        assert matcher.matches_pattern('src/main.py', patterns, None)

    def test_exclude_patterns_loaded_from_yaml(self):
        """YAMLからexclude_patternsが正しく読み込まれる"""
        with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml', delete=False) as f:
            rules_data = {
                'rules': [
                    {
                        'name': '全ファイル禁止（例外あり）',
                        'patterns': ['**/*'],
                        'severity': 'deny',
                        'scope': 'pmo',
                        'exclude_patterns': ['vibes/docs/**'],
                        'message': 'ファイル編集禁止（vibes/docs除く）'
                    }
                ]
            }
            yaml.dump(rules_data, f)
            temp_path = Path(f.name)

        try:
            matcher = FileConventionMatcher(temp_path)
            assert len(matcher.rules) == 1
            assert matcher.rules[0].exclude_patterns == ['vibes/docs/**']

            # 除外パスはルールに非該当
            rules = matcher.check_file('vibes/docs/rules/test.md')
            assert len(rules) == 0

            # 除外外パスはルールに該当
            rules = matcher.check_file('src/main.py')
            assert len(rules) == 1
            assert rules[0].name == '全ファイル禁止（例外あり）'
        finally:
            temp_path.unlink()

    def test_exclude_patterns_default_empty(self):
        """exclude_patterns未指定時はデフォルト空リスト"""
        with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml', delete=False) as f:
            rules_data = {
                'rules': [
                    {
                        'name': 'ルール（exclude未指定）',
                        'patterns': ['**/*.py'],
                        'severity': 'warn',
                        'message': 'テスト'
                    }
                ]
            }
            yaml.dump(rules_data, f)
            temp_path = Path(f.name)

        try:
            matcher = FileConventionMatcher(temp_path)
            assert matcher.rules[0].exclude_patterns == []
            # 既存動作に影響なし
            rules = matcher.check_file('src/main.py')
            assert len(rules) == 1
        finally:
            temp_path.unlink()


class TestAcceptanceExcludePatterns:
    """受入テスト: exclude_patternsによるvibes/docs配下の除外動作確認
    
    #8187 コミット ed238e5 で導入されたexclude_patterns機能の受入テスト。
    scope=tech-lead/pmoのdenyルールがvibes/docs配下を正しく除外するか検証。
    """

    @pytest.fixture
    def deny_matcher(self):
        """tech-lead/pmoスコープのdenyルール（vibes/docs除外）を持つマッチャー"""
        with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml', delete=False) as f:
            rules_data = {
                'rules': [
                    {
                        'name': 'vibes/docs外ファイル編集禁止',
                        'patterns': ['**/*'],
                        'severity': 'deny',
                        'scope': 'tech-lead',
                        'exclude_patterns': ['vibes/docs/**'],
                        'message': 'vibes/docs配下以外のファイル編集は禁止です'
                    },
                    {
                        'name': 'PMO: vibes/docs外ファイル編集禁止',
                        'patterns': ['**/*'],
                        'severity': 'deny',
                        'scope': 'pmo',
                        'exclude_patterns': ['vibes/docs/**'],
                        'message': 'PMO: vibes/docs配下以外のファイル編集は禁止です'
                    }
                ]
            }
            yaml.dump(rules_data, f)
            temp_path = Path(f.name)

        matcher = FileConventionMatcher(temp_path)
        yield matcher
        temp_path.unlink()

    def test_tc1_deny_outside_vibes_docs(self, deny_matcher):
        """TC1: vibes/docs外ファイル編集がdenyされること
        
        scope=tech-lead/pmoのdenyルールがvibes/docs外ファイルに適用される。
        """
        # src配下のファイル → denyルール適用
        rules = deny_matcher.check_file('src/domain/services/file_convention_matcher.py')
        assert len(rules) == 2, f"vibes/docs外ファイルにdenyルールが適用されるべき: got {len(rules)}"
        for rule in rules:
            assert rule.severity == 'deny'

    def test_tc2_allow_vibes_docs(self, deny_matcher):
        """TC2: vibes/docs配下ファイル編集がallowされること
        
        vibes/docs配下パスはexclude_patternsにより除外され、ルール非適用。
        """
        # vibes/docs配下 → ルール非適用（除外）
        rules = deny_matcher.check_file('vibes/docs/rules/documentation_standards.md')
        assert len(rules) == 0, f"vibes/docs配下はルール非適用のはず: got {len(rules)}"

        rules = deny_matcher.check_file('vibes/docs/README.md')
        assert len(rules) == 0, f"vibes/docs/README.mdはルール非適用のはず: got {len(rules)}"

    def test_tc3_deep_path_exclude(self, deny_matcher):
        """TC3: 深いパスのexclude_patterns動作確認
        
        vibes/docs/rules/nested/deep/test.md のような深い階層でも除外が有効。
        """
        rules = deny_matcher.check_file('vibes/docs/rules/nested/deep/test.md')
        assert len(rules) == 0, f"深いパスでも除外が有効のはず: got {len(rules)}"

        rules = deny_matcher.check_file('vibes/docs/specs/sub/another/file.yaml')
        assert len(rules) == 0, f"深いパスでも除外が有効のはず: got {len(rules)}"

    def test_tc4_deny_outside_vibes_docs_various(self, deny_matcher):
        """TC4: vibes/docs外の引き続きdeny確認
        
        README.md, tests/配下等、vibes/docs外のファイルは引き続きdeny適用。
        """
        # README.md → deny適用
        rules = deny_matcher.check_file('README.md')
        assert len(rules) == 2, f"README.mdにdenyルール適用されるべき: got {len(rules)}"
        for rule in rules:
            assert rule.severity == 'deny'

        # tests/配下 → deny適用
        rules = deny_matcher.check_file('tests/test_file_convention_matcher.py')
        assert len(rules) == 2, f"tests/配下にdenyルール適用されるべき: got {len(rules)}"
        for rule in rules:
            assert rule.severity == 'deny'

        # vibes/だがdocs/ではないパス → deny適用
        rules = deny_matcher.check_file('vibes/other/file.md')
        assert len(rules) == 2, f"vibes/docs外のvibes/配下にもdeny適用されるべき: got {len(rules)}"



#!/usr/bin/env python3
"""install-hooks コマンド実装

機能:
1. .claude-nagger/ ディレクトリがなければ雛形生成
2. .claude/settings.json へ PreToolUse フック設定を登録
"""

import json
import os
import sys
from pathlib import Path
from typing import Any


class InstallHooksCommand:
    """フックインストールコマンド"""

    # 雛形ファイルの内容
    FILE_CONVENTIONS_TEMPLATE = """\
# ファイル編集規約定義
# 特定のファイルパターンに対して適用される規約を定義します

rules:
  # 例: View層編集規約
  # - name: "View層編集規約"
  #   patterns:
  #     - "**/app/views/**/*.erb"
  #   severity: "block"  # block: 完全ブロック, warn: 警告のみ
  #   token_threshold: 35000
  #   message: |
  #     このファイルを変更する場合は規約を確認してください
  []
"""

    COMMAND_CONVENTIONS_TEMPLATE = """\
# コマンド実行規約定義
# 危険なコマンドや確認が必要なコマンドに対して適用される規約を定義します

rules:
  # 例: Git操作規約
  # - name: "Git操作規約"
  #   patterns:
  #     - "git*"
  #   severity: "block"
  #   token_threshold: 25000
  #   message: |
  #     Git操作を実行する場合は規約を確認してください
  []
"""

    MCP_CONVENTIONS_TEMPLATE = """\
# MCP規約設定
# MCPツール呼び出し時のパターンマッチ規約を定義します
# tool_pattern: 正規表現（re.match）でMCPツール名にマッチ

rules:
  # 例: Redmine更新操作規約
  # - name: "Redmine更新確認"
  #   tool_pattern: "mcp__redmine_epic_grid__update.*"
  #   severity: "warn"
  #   token_threshold: 25000
  #   message: |
  #     Redmineの更新操作を行います。確認してください。
  []
"""

    CONFIG_TEMPLATE = """\
# claude-nagger 設定ファイル

# セッション開始時設定
session_startup:
  enabled: true
  messages:
    first_time:
      title: "プロジェクトセットアップ"
      main_text: |
        プロジェクトの規約を確認してください
      severity: "block"
    # 継続セッション用（token_threshold超過時に表示）
    repeated:
      title: "規約再確認"
      main_text: |
        規約を再確認してください
      severity: "block"

  # subagent別override設定（subagent自身に直接ブロッキング通知される）
  # 仕組み: SubagentStart hook → マーカー作成 → 次のPreToolUseでsubagentをブロック
  # 解決順序: base(上記) → subagent_default → subagent_types.{type}
  # suffix対応: "coder-fix-bug" → "coder" にフォールバック
  # 命名規約: Agent Teamsメンバーのnameはsubagent_typesのキーと一致させること
  #   例: name="coder" → subagent_types.coder にマッチ
  overrides:
    subagent_default:
      messages:
        first_time:
          title: "subagent規約"
          main_text: |
            [ ] スコープ外のファイルを編集しないこと
            [ ] 作業完了後に結果を報告すること
    subagent_types:
      Explore:
        enabled: false
      Plan:
        enabled: false
      Bash:
        messages:
          first_time:
            title: "Bash subagent規約"
            main_text: |
              [ ] 破壊的コマンド禁止
      scribe:
        messages:
          first_time:
            title: "scribe subagent規約"
            main_text: |
              基本規約:
              [ ] 指定された操作のみ行うこと
              [ ] スコープ外の操作は行わないこと
              報告規約:
              [ ] 作業完了後に結果を報告すること
              [ ] コメント冒頭に [scribe] プレフィックスを付けること
      tester:
        messages:
          first_time:
            title: "tester subagent規約"
            main_text: |
              基本規約:
              [ ] 仕様からテストを設計すること（実装詳細に依存しない）
              [ ] コードを編集しないこと（Read専用）
              [ ] テスト失敗時に自分で修正しないこと
              報告規約:
              [ ] テスト結果を報告すること
              [ ] 失敗時は期待動作 vs 実際の動作を明記すること
              [ ] コメント冒頭に [tester] プレフィックスを付けること
      coder:
        messages:
          first_time:
            title: "coder subagent規約"
            main_text: |
              基本規約:
              [ ] 指示されたスコープ外のファイルを編集しないこと
              [ ] 判断が必要な場合は実装せず報告すること
              [ ] テストがあれば実行してから完了報告すること
              報告規約:
              [ ] 完了時に変更内容・懸念事項を報告すること
              [ ] コメント冒頭に [coder] プレフィックスを付けること
      conductor:
        messages:
          first_time:
            title: "conductor subagent規約"
            main_text: |
              [ ] 直接作業を実行しないこと（コード編集・ファイル操作禁止）
              [ ] 実作業はSubAgent（coder/tester/scribe）へ委譲すること
              [ ] SubAgentの成果物を必ずレビューすること

# SendMessageガード設定（Redmine基盤通信強制）
# 毎回検査。違反時のみblock、合格時は無通知通過
sendmessage_guard:
  enabled: true
  pattern: '^issue_\\d+ \\[.+\\]$'
  exempt_types:
    - shutdown_request
    - shutdown_response
    - plan_approval_response

# Discord通知設定
# webhook_url/thread_id は secrets.yaml から自動解決（${VAR} → secrets.yaml discord.* にフォールバック）
notifications:
  discord:
    enabled: true
    webhook_url: "${DISCORD_WEBHOOK_URL}"
    thread_id: "${DISCORD_THREAD_ID}"

# トランスクリプト保存設定（raw modeトランスクリプトDB保存）
transcript_storage:
  enabled: false
  # mode: raw=生データ格納 / indexed=インデックス付き / structured=構造化格納
  mode: "structured"
  retention_days: 30

# コンテキスト管理設定
context_management:
  reminder_thresholds:
    light_warning: 30000
    medium_warning: 60000
    critical_warning: 100000

# role判定: agent_typeの前方一致でroleを確定（最長一致優先）
# suffix多様性を吸収（coder-fix-bug → coder, pmo-2 → pmo）
# trusted_prefixesにマッチしない場合はtask_spawnsフォールバック
role_resolution:
  trusted_prefixes: {}
  #  "coder": "coder"
  #  "pmo": "pmo"
  #  "tech-lead": "tech-lead"
  #  "tester": "tester"
  #  "researcher": "researcher"

# デバッグ設定
debug:
  enable_logging: false
"""

    SECRETS_TEMPLATE = """\
# 機密情報設定ファイル
# このファイルはGit管理対象外です（.gitignoreで除外済み）

# Discord通知設定
discord:
  webhook_url: ""
  thread_id: ""

# その他の機密情報
# api_keys:
#   service_name: "your-api-key"
"""

    GITIGNORE_TEMPLATE = """\
# vault/ディレクトリ内の全ファイルを除外
# 機密情報の漏洩防止
*
!.gitignore
"""

    # .claude-nagger/直下の.gitignoreテンプレート
    DOTCN_GITIGNORE_TEMPLATE = """\
# ランタイム生成ファイル（自動生成: claude-nagger install-hooks）
state.db
state.db-wal
state.db-shm
suggested_rules/
"""

    # デフォルトのPreToolUseフック設定
    # サブコマンド形式（claude-nagger hook <name>）を使用
    # uv tool install / pip install 両環境で動作
    DEFAULT_PRETOOLUSE_HOOKS = [
        {
            "matcher": "",
            "hooks": [
                {
                    "type": "command",
                    "command": "claude-nagger hook session-startup"
                }
            ]
        },
        {
            "matcher": "mcp__.*__write.*",
            "hooks": [
                {
                    "type": "command",
                    "command": "claude-nagger hook implementation-design"
                }
            ]
        },
        {
            "matcher": "mcp__.*replace.*",
            "hooks": [
                {
                    "type": "command",
                    "command": "claude-nagger hook implementation-design"
                }
            ]
        },
        {
            "matcher": "mcp__.*insert.*",
            "hooks": [
                {
                    "type": "command",
                    "command": "claude-nagger hook implementation-design"
                }
            ]
        },
        {
            "matcher": "Bash",
            "hooks": [
                {
                    "type": "command",
                    "command": "claude-nagger hook implementation-design"
                }
            ]
        },
        {
            "matcher": "Edit",
            "hooks": [
                {
                    "type": "command",
                    "command": "claude-nagger hook implementation-design"
                }
            ]
        },
        {
            "matcher": "Write",
            "hooks": [
                {
                    "type": "command",
                    "command": "claude-nagger hook implementation-design"
                }
            ]
        },
        {
            "matcher": "MultiEdit",
            "hooks": [
                {
                    "type": "command",
                    "command": "claude-nagger hook implementation-design"
                }
            ]
        },
        {
            "matcher": "SendMessage",
            "hooks": [
                {
                    "type": "command",
                    "command": "claude-nagger hook sendmessage-guard"
                }
            ]
        },
    ]

    # デフォルトのNotificationフック設定
    # 通知発生時（複数API呼び出し間等）に実行される
    # claude-nagger notify コマンドを使用（uv/pip両環境対応）
    DEFAULT_NOTIFICATION_HOOKS: list = [
        {
            "matcher": "",
            "hooks": [
                {
                    "type": "command",
                    "command": "claude-nagger notify \"📢 Claude Code から通知があります\""
                }
            ]
        }
    ]

    # デフォルトのStopフック設定
    # タスク完了・停止時に実行される
    # claude-nagger notify コマンドを使用（uv/pip両環境対応）
    DEFAULT_STOP_HOOKS: list = [
        {
            "matcher": "",
            "hooks": [
                {
                    "type": "command",
                    "command": "claude-nagger notify \"🛑 Claude Code セッションが停止されました\""
                }
            ]
        },
        {
            "matcher": "",
            "hooks": [
                {
                    "type": "command",
                    "command": "claude-nagger hook suggest-rules-trigger"
                }
            ]
        },
        {
            "matcher": "",
            "hooks": [
                {
                    "type": "command",
                    "command": "claude-nagger hook transcript-storage"
                }
            ]
        }
    ]

    # デフォルトのSessionStartフック設定
    # compact発生時にマーカーをリセットし、既存フローを再発火させる
    DEFAULT_SESSIONSTART_HOOKS: list = [
        {
            "matcher": "compact",
            "hooks": [
                {
                    "type": "command",
                    "command": "claude-nagger hook compact-detected"
                }
            ]
        }
    ]


    # デフォルトのSubagentStartフック設定
    # サブエージェント開始時にマーカーファイルを作成
    DEFAULT_SUBAGENTSTART_HOOKS: list = [
        {
            "matcher": "",
            "hooks": [
                {
                    "type": "command",
                    "command": "claude-nagger hook subagent-event"
                }
            ]
        }
    ]

    # デフォルトのSubagentStopフック設定
    # サブエージェント終了時にマーカーファイルを削除
    DEFAULT_SUBAGENTSTOP_HOOKS: list = [
        {
            "matcher": "",
            "hooks": [
                {
                    "type": "command",
                    "command": "claude-nagger hook subagent-event"
                }
            ]
        }
    ]

    # デフォルトのPostToolUseフック設定
    # Redmine MCPツール操作後にDiscord通知を送信
    DEFAULT_POSTTOOLUSE_HOOKS: list = [
        {
            "matcher": "mcp__redmine_epic_grid__.*",
            "hooks": [
                {
                    "type": "command",
                    "command": "claude-nagger hook redmine-discord"
                }
            ]
        }
    ]

    def __init__(self, force: bool = False, dry_run: bool = False):
        """
        Args:
            force: 既存ファイルを上書きするか
            dry_run: 実行内容を表示するのみ
        """
        self.force = force
        self.dry_run = dry_run
        self.project_root = Path.cwd()

    def execute(self) -> int:
        """コマンド実行"""
        print("claude-nagger install-hooks")
        print("=" * 40)

        try:
            # 1. .claude-nagger/ ディレクトリと雛形ファイル生成
            self._create_claude_nagger_dir()

            # 2. .claude/settings.json へフック設定追加
            self._update_settings_json()

            print()
            print("インストール完了")
            return 0

        except Exception as e:
            print(f"エラー: {e}")
            return 1

    def _create_claude_nagger_dir(self):
        """`.claude-nagger/` ディレクトリと雛形ファイルを生成"""
        nagger_dir = self.project_root / ".claude-nagger"

        if self.dry_run:
            print(f"[dry-run] ディレクトリ作成: {nagger_dir}")
        else:
            nagger_dir.mkdir(exist_ok=True)
            print(f"ディレクトリ確認: {nagger_dir}")

        # 雛形ファイル生成
        files = {
            "config.yaml": self.CONFIG_TEMPLATE,
            "file_conventions.yaml": self.FILE_CONVENTIONS_TEMPLATE,
            "command_conventions.yaml": self.COMMAND_CONVENTIONS_TEMPLATE,
            "mcp_conventions.yaml": self.MCP_CONVENTIONS_TEMPLATE,
        }

        for filename, content in files.items():
            file_path = nagger_dir / filename
            self._write_file(file_path, content)

        # .claude-nagger/.gitignore 生成（ランタイムファイル除外用）
        self._write_file(nagger_dir / ".gitignore", self.DOTCN_GITIGNORE_TEMPLATE)

        # vault/ ディレクトリと機密ファイル生成
        self._create_vault_dir(nagger_dir)

    def _create_vault_dir(self, nagger_dir: Path):
        """vault/ディレクトリと機密ファイルを生成"""
        vault_dir = nagger_dir / "vault"

        if self.dry_run:
            print(f"[dry-run] ディレクトリ作成: {vault_dir}")
        else:
            vault_dir.mkdir(exist_ok=True)
            print(f"ディレクトリ確認: {vault_dir}")

        # vault内ファイル生成
        vault_files = {
            "secrets.yaml": self.SECRETS_TEMPLATE,
            ".gitignore": self.GITIGNORE_TEMPLATE,
        }

        for filename, content in vault_files.items():
            file_path = vault_dir / filename
            self._write_file(file_path, content)

    def _write_file(self, path: Path, content: str):
        """ファイル書き込み（force/dry-run考慮）"""
        if path.exists() and not self.force:
            print(f"  スキップ（既存）: {path.name}")
            return

        if self.dry_run:
            action = "上書き" if path.exists() else "作成"
            print(f"  [dry-run] {action}: {path.name}")
        else:
            path.write_text(content, encoding="utf-8")
            action = "上書き" if path.exists() else "作成"
            print(f"  {action}: {path.name}")

    def _update_settings_json(self):
        """`.claude/settings.json` にフック設定を追加"""
        claude_dir = self.project_root / ".claude"
        settings_path = claude_dir / "settings.json"

        print()
        print("フック設定更新:")

        # .claude/ ディレクトリ作成
        if self.dry_run:
            if not claude_dir.exists():
                print(f"  [dry-run] ディレクトリ作成: {claude_dir}")
        else:
            claude_dir.mkdir(exist_ok=True)

        # 既存設定の読み込み
        settings = self._load_settings(settings_path)

        # 各フック設定のマージ
        hook_configs = [
            ("PreToolUse", self.DEFAULT_PRETOOLUSE_HOOKS),
            ("Notification", self.DEFAULT_NOTIFICATION_HOOKS),
            ("Stop", self.DEFAULT_STOP_HOOKS),
            ("SessionStart", self.DEFAULT_SESSIONSTART_HOOKS),
            ("SubagentStart", self.DEFAULT_SUBAGENTSTART_HOOKS),
            ("SubagentStop", self.DEFAULT_SUBAGENTSTOP_HOOKS),
            ("PostToolUse", self.DEFAULT_POSTTOOLUSE_HOOKS),
        ]

        any_updated = False
        for hook_type, default_hooks in hook_configs:
            if default_hooks:  # 空リストはスキップ
                updated = self._merge_hook_entries(settings, hook_type, default_hooks)
                any_updated = any_updated or updated

        if not any_updated:
            print("  フック設定は既に存在します（変更なし）")
            return

        # 設定の書き込み
        if self.dry_run:
            print("  [dry-run] settings.json を更新")
        else:
            self._save_settings(settings_path, settings)
            print(f"  更新: {settings_path}")

    def _load_settings(self, path: Path) -> dict[str, Any]:
        """settings.json を読み込み"""
        if not path.exists():
            return {}

        try:
            content = path.read_text(encoding="utf-8")
            return json.loads(content)
        except json.JSONDecodeError:
            print(f"  警告: {path} のパースに失敗。新規作成します。")
            return {}

    def _merge_pretooluse_hooks(self, settings: dict[str, Any]) -> bool:
        """PreToolUseフックをマージ（後方互換性のためのラッパー）

        Returns:
            bool: 変更があった場合True
        """
        return self._merge_hook_entries(settings, "PreToolUse", self.DEFAULT_PRETOOLUSE_HOOKS)

    def _merge_hook_entries(
        self,
        settings: dict[str, Any],
        hook_type: str,
        default_hooks: list[dict[str, Any]]
    ) -> bool:
        """指定タイプのフックをマージ（汎用）

        Args:
            settings: 設定dict
            hook_type: フックタイプ (PreToolUse/Notification/Stop)
            default_hooks: 追加するデフォルトフック

        Returns:
            bool: 変更があった場合True
        """
        if "hooks" not in settings:
            settings["hooks"] = {}

        hooks = settings["hooks"]

        if hook_type not in hooks:
            hooks[hook_type] = []

        hook_list = hooks[hook_type]

        # 既存の(matcher, command)ペアを収集
        existing_entries = set()
        for hook_entry in hook_list:
            matcher = hook_entry.get("matcher", "")
            for hook in hook_entry.get("hooks", []):
                if "command" in hook:
                    existing_entries.add((matcher, hook["command"]))

        # 新規フックを追加（matcher+commandの組み合わせで重複回避）
        added = False
        for new_entry in default_hooks:
            matcher = new_entry.get("matcher", "")
            for hook in new_entry.get("hooks", []):
                cmd = hook.get("command", "")
                if cmd and (matcher, cmd) not in existing_entries:
                    hook_list.append(new_entry)
                    existing_entries.add((matcher, cmd))
                    added = True
                    print(f"  追加 [{hook_type}]: {cmd}")
                elif cmd:
                    print(f"  スキップ（既存）[{hook_type}]: {cmd}")

        return added

    def _save_settings(self, path: Path, settings: dict[str, Any]):
        """settings.json を保存"""
        content = json.dumps(settings, indent=2, ensure_ascii=False)
        path.write_text(content + "\n", encoding="utf-8")


def ensure_config_exists(project_root: Path = None) -> bool:
    """設定ファイルの存在を保証する（冪等）
    
    .claude-nagger/ディレクトリと設定ファイル群が存在しない場合は生成する。
    フック実行時に自動的に呼び出される。
    
    Args:
        project_root: プロジェクトルートパス（デフォルト: カレントディレクトリ）
    
    Returns:
        bool: ファイルが生成された場合True、既存の場合False
    """
    if project_root is None:
        project_root = Path.cwd()
    
    nagger_dir = project_root / ".claude-nagger"
    config_path = nagger_dir / "config.yaml"
    
    # 設定ファイルが存在すれば何もしない
    if config_path.exists():
        return False
    
    # 設定ファイルを生成
    generated = False
    
    # ディレクトリ作成
    if not nagger_dir.exists():
        nagger_dir.mkdir(exist_ok=True)
        generated = True
    
    # 各ファイルを不足分のみ生成
    files = {
        "config.yaml": InstallHooksCommand.CONFIG_TEMPLATE,
        "file_conventions.yaml": InstallHooksCommand.FILE_CONVENTIONS_TEMPLATE,
        "command_conventions.yaml": InstallHooksCommand.COMMAND_CONVENTIONS_TEMPLATE,
        "mcp_conventions.yaml": InstallHooksCommand.MCP_CONVENTIONS_TEMPLATE,
    }
    
    for filename, content in files.items():
        file_path = nagger_dir / filename
        if not file_path.exists():
            file_path.write_text(content, encoding="utf-8")
            generated = True
    
    # vault/ディレクトリと機密ファイル生成
    vault_dir = nagger_dir / "vault"
    if not vault_dir.exists():
        vault_dir.mkdir(exist_ok=True)
        generated = True
    
    vault_files = {
        "secrets.yaml": InstallHooksCommand.SECRETS_TEMPLATE,
        ".gitignore": InstallHooksCommand.GITIGNORE_TEMPLATE,
    }
    
    for filename, content in vault_files.items():
        file_path = vault_dir / filename
        if not file_path.exists():
            file_path.write_text(content, encoding="utf-8")
            generated = True

    # .claude-nagger/.gitignore 生成（ランタイムファイル除外用）
    gitignore_path = nagger_dir / ".gitignore"
    if not gitignore_path.exists():
        gitignore_path.write_text(InstallHooksCommand.DOTCN_GITIGNORE_TEMPLATE, encoding="utf-8")
        generated = True

    # 自動生成時の警告出力
    if generated:
        print("警告: 設定ファイルを自動生成しました (.claude-nagger/)", file=sys.stderr)
    
    return generated



"""SendMessage ガードフック

SendMessage ツール使用時にメッセージ内容を検査し、
Redmine基盤通信規約（issue_id + 短文のみ）を強制する PreToolUse hook。
"""

import logging
import re
import sys
import time
from pathlib import Path
from typing import Dict, Any, Optional

sys.path.append(str(Path(__file__).parent.parent.parent))

from domain.hooks.base_hook import BaseHook, ExitCode
from domain.services.caller_role_service import get_caller_roles
from infrastructure.config.config_manager import ConfigManager

# デフォルト設定
DEFAULT_PATTERN = r"^.+$"
DEFAULT_EXEMPT_TYPES = [
    "shutdown_request",
    "shutdown_response",
    "plan_approval_response",
]

# block reason テンプレート
DEFAULT_BLOCK_REASON_TEMPLATE = """\
SendMessage規約違反
━━━━━━━━━━━━━━━━━
違反: {violation}
対処: 設定されたフォーマットに従ってSendMessageを再送してください\
"""

# P2P違反メッセージデフォルト
DEFAULT_P2P_BROADCAST_BLOCK_MESSAGE = "P2P制御: role={roles} はbroadcast禁止。team-leadへ個別送信してください"
DEFAULT_P2P_MESSAGE_BLOCK_MESSAGE = "P2P制御: role={roles} から {recipient} への直接通信は禁止。team-leadを経由してください"


class SendMessageGuardHook(BaseHook):
    """SendMessage ツール使用時のメッセージ内容検査フック

    セッションマーカーは使用しない（毎回のSendMessageを検査するため）。
    """

    def __init__(self, debug: Optional[bool] = None):
        """初期化

        Args:
            debug: デバッグモードフラグ
        """
        super().__init__(debug=debug)
        self._config_manager = ConfigManager()
        self._guard_config = self._load_guard_config()

    def _load_guard_config(self) -> Dict[str, Any]:
        """sendmessage_guard 設定を読み込み

        config.yaml の sendmessage_guard セクションを取得。
        未定義の場合はデフォルト値を使用。

        Returns:
            ガード設定辞書
        """
        raw = self._config_manager.config.get("sendmessage_guard", {})
        config = {
            "enabled": raw.get("enabled", True),
            "pattern": raw.get("pattern", DEFAULT_PATTERN),
            "exempt_types": raw.get("exempt_types", DEFAULT_EXEMPT_TYPES),
        }
        # block_message: 設定されていればカスタムテンプレートを使用
        if "block_message" in raw:
            config["block_message"] = raw["block_message"]

        # P2P通信制御ルール
        p2p_raw = raw.get("p2p_rules", {})
        config["p2p_rules"] = {
            "enabled": p2p_raw.get("enabled", False),
            "default_policy": p2p_raw.get("default_policy", "deny"),
            "broadcast_allowed_roles": p2p_raw.get("broadcast_allowed_roles", []),
            "matrix": p2p_raw.get("matrix", {}),
            "broadcast_block_message": p2p_raw.get(
                "broadcast_block_message", DEFAULT_P2P_BROADCAST_BLOCK_MESSAGE
            ),
            "message_block_message": p2p_raw.get(
                "message_block_message", DEFAULT_P2P_MESSAGE_BLOCK_MESSAGE
            ),
        }
        return config

    # --- 判定ロジック（テスト容易性のためメソッド切り出し） ---

    def is_target_tool(self, tool_name: str) -> bool:
        """SendMessage ツールかどうかを判定

        Args:
            tool_name: ツール名

        Returns:
            SendMessage ツールの場合 True
        """
        return tool_name == "SendMessage"

    def is_exempt_type(self, message_type: str) -> bool:
        """免除対象メッセージタイプかどうかを判定

        Args:
            message_type: メッセージタイプ

        Returns:
            免除対象の場合 True
        """
        return message_type in self._guard_config["exempt_types"]

    def validate_content(self, content: str) -> Dict[str, Any]:
        """メッセージ内容を検証

        正規表現パターンで完全一致チェック。

        Args:
            content: メッセージ内容

        Returns:
            {"valid": bool, "violation": str or None}
        """
        pattern = self._guard_config["pattern"]

        # フォーマット不一致チェック（正規表現で完全一致）
        if not re.match(pattern, content):
            return {
                "valid": False,
                "violation": f"フォーマット不一致。設定パターン: {pattern}",
            }

        return {"valid": True, "violation": None}

    # --- P2P通信制御 ---

    def _validate_p2p(self, input_data: Dict[str, Any], tool_input: Dict[str, Any]) -> Dict[str, Any]:
        """P2P通信許可を検証

        P2Pマトリクスに基づき、callerのroleからrecipientへの通信可否を判定。
        caller/recipient両方を正規化してmatrix照合する。

        Returns:
            {"valid": True} or {"valid": False, "violation": "..."}
        """
        p2p_rules = self._guard_config.get("p2p_rules", {})

        # P2P無効時はスキップ
        if not p2p_rules.get("enabled", False):
            return {"valid": True}

        # exempt_types はスキップ
        message_type = tool_input.get("type", "")
        if message_type in self._guard_config["exempt_types"]:
            return {"valid": True}

        # caller roles 取得
        logger = logging.getLogger(__name__)
        roles = get_caller_roles(input_data, logger=logger)

        # role不明 → allow（安全側: leader or 未識別）
        if not roles:
            return {"valid": True}

        # leader は全recipient許可
        if "leader" in roles:
            return {"valid": True}

        # broadcast 判定
        if message_type == "broadcast":
            broadcast_allowed = p2p_rules.get("broadcast_allowed_roles", [])
            if not any(role in broadcast_allowed for role in roles):
                # config管理化テンプレート使用
                msg_template = p2p_rules.get(
                    "broadcast_block_message", DEFAULT_P2P_BROADCAST_BLOCK_MESSAGE
                )
                return {
                    "valid": False,
                    "violation": msg_template.format(roles=roles),
                }
            return {"valid": True}

        # message 判定
        if message_type == "message":
            recipient_raw = tool_input.get("recipient", "") or ""
            # recipientを正規化してmatrix照合（issue: raw値のまま照合されるバグ修正）
            from infrastructure.db.subagent_repository import _normalize_role, _get_known_roles_from_config
            known_roles = _get_known_roles_from_config()
            recipient = _normalize_role(recipient_raw, known_roles) if known_roles else recipient_raw
            matrix = p2p_rules.get("matrix", {})
            default_policy = p2p_rules.get("default_policy", "deny")
            for role in roles:
                allowed_recipients = matrix.get(role, [])
                if recipient in allowed_recipients:
                    return {"valid": True}
            # matrixに該当なし → default_policyで判定
            if default_policy == "allow":
                return {"valid": True}
            # config管理化テンプレート使用
            msg_template = p2p_rules.get(
                "message_block_message", DEFAULT_P2P_MESSAGE_BLOCK_MESSAGE
            )
            return {
                "valid": False,
                "violation": msg_template.format(roles=roles, recipient=recipient),
            }

        # その他のtype → allow
        return {"valid": True}

    # --- BaseHook 抽象メソッドの実装 ---

    def should_process(self, input_data: Dict[str, Any]) -> bool:
        """処理対象かどうかを判定

        1. tool_name が SendMessage でなければ False
        2. tool_input.type が exempt_types に含まれれば False
        3. それ以外 True

        Args:
            input_data: 入力データ

        Returns:
            処理対象の場合 True
        """
        tool_name = input_data.get("tool_name", "")
        if not self.is_target_tool(tool_name):
            return False

        tool_input = input_data.get("tool_input", {})
        message_type = tool_input.get("type", "")
        if self.is_exempt_type(message_type):
            self.log_debug(f"Exempt type: {message_type}")
            return False

        return True

    def process(self, input_data: Dict[str, Any]) -> Dict[str, str]:
        """メッセージ内容を検査し、規約違反をブロック

        P2P通信制御 → content検証の順で実行。

        Args:
            input_data: 入力データ

        Returns:
            decision と reason を含む辞書
        """
        tool_input = input_data.get("tool_input", {})

        # P2P通信制御（content検証より先に実行）
        p2p_result = self._validate_p2p(input_data, tool_input)
        if not p2p_result["valid"]:
            self.log_info(f"BLOCK(P2P): {p2p_result['violation']}")
            return {
                "decision": "block",
                "reason": p2p_result["violation"],
                "skip_warn_only": True,  # P2Pはセキュリティ制約: WARN_ONLY迂回不可
            }

        # 既存content検証
        content = tool_input.get("content", "")

        result = self.validate_content(content)

        if not result["valid"]:
            # block_message テンプレート: config指定があればそちらを使用
            template = self._guard_config.get(
                "block_message", DEFAULT_BLOCK_REASON_TEMPLATE
            )
            reason = template.format(
                violation=result["violation"],
                pattern=self._guard_config["pattern"],
            )
            self.log_info(f"BLOCK: {result['violation']}")
            return {"decision": "block", "reason": reason}

        self.log_debug(f"APPROVE: content validated")
        return {"decision": "approve", "reason": ""}

    # --- run() オーバーライド ---

    def run(self) -> int:
        """フックのメインエントリーポイント

        BaseHook.run() のセッションマーカー処理をスキップする簡略版。
        毎回の SendMessage を検査するためマーカーは不要。

        Returns:
            ExitCode
        """
        self._start_time = time.time()

        # 設定ファイル存在保証
        from application.install_hooks import ensure_config_exists
        ensure_config_exists()

        self._structured_logger.log_hook_event(
            event_type="start",
            hook_name=self.__class__.__name__,
            debug_mode=self.debug,
        )

        from shared.permission_mode import PermissionModeBehavior
        self._current_permission_mode_behavior: Optional[PermissionModeBehavior] = None

        try:
            input_data = self.read_input()
            if not input_data:
                self.log_debug("No input data, exiting")
                return ExitCode.SUCCESS

            # permission_mode によるスキップ判定
            should_skip, behavior = self.should_skip_by_permission_mode(input_data)
            self._current_permission_mode_behavior = behavior
            if should_skip:
                self.log_info(f"Skipping due to permission_mode behavior: {behavior}")
                return ExitCode.SUCCESS

            # 処理対象チェック（セッションマーカーチェックなし）
            if not self.should_process(input_data):
                self.log_debug("Not a target for processing, skipping")
                return ExitCode.SUCCESS

            # 処理実行
            result = self.process(input_data)

            # WARN_ONLY モード: block を allow に変換（skip_warn_only=True時は変換しない）
            decision = result["decision"]
            if behavior == PermissionModeBehavior.WARN_ONLY and decision == "block" and not result.get("skip_warn_only"):
                self.log_info("Converting block to allow due to WARN_ONLY mode")
                decision = "approve"
                original_reason = result.get("reason", "")
                result["reason"] = f"[WARN_ONLY] {original_reason}" if original_reason else "[WARN_ONLY]"

            if self.output_response(decision, result.get("reason", "")):
                self._log_hook_end(decision=decision, reason=result.get("reason", ""))
                return ExitCode.SUCCESS
            else:
                self._log_hook_end(decision="output_error")
                return ExitCode.ERROR

        except Exception as e:
            self.log_error(f"Unexpected error in run", error=str(e))
            self._log_hook_end(decision="error")
            return ExitCode.ERROR


def main():
    """メインエントリーポイント"""
    hook = SendMessageGuardHook(debug=False)
    sys.exit(hook.run())


if __name__ == "__main__":
    main()



"""caller role取得サービス（issue_7155, issue_7352）

ImplementationDesignHook._get_caller_roles()を独立関数として抽出。
SendMessageGuardHook等からも利用可能。
issue_7352: agent_idベースに移行。transcript走査を全廃。
"""

import logging
from typing import Optional

from shared.trusted_prefixes import resolve_trusted_prefix


def get_caller_roles(
    input_data: dict,
    logger: Optional[logging.Logger] = None,
) -> set:
    """現在のcaller（subagent）のroleセットを取得

    agent_idベース: input_data['agent_id']からDB検索し、roleを返す。
    agent_id不在時はleader判定（is_leader_tool_use()と一貫）。

    Args:
        input_data: 入力データ（agent_id, session_id等を含む）
        logger: ロガー（Noneの場合は__name__で取得）

    Returns:
        正規化済みroleのセット
    """
    if logger is None:
        logger = logging.getLogger(__name__)

    # agent_idベースのagent特定（優先: issue_7352）
    from domain.services.leader_detection import find_caller_agent_id
    agent_id = find_caller_agent_id(input_data)
    if agent_id:
        try:
            from infrastructure.db import NaggerStateDB, SubagentRepository
            db = NaggerStateDB(NaggerStateDB.resolve_db_path())
            subagent_repo = SubagentRepository(db)
            record = subagent_repo.get(agent_id)
            db.close()
            # trusted_prefixes照合: agent_typeから直接roleを確定（race condition回避）
            if record and record.agent_type:
                trusted_role = resolve_trusted_prefix(record.agent_type, logger)
                if trusted_role:
                    logger.info(
                        f"CALLER ROLES (trusted_prefix): agent_id={agent_id}, "
                        f"agent_type={record.agent_type}, role={trusted_role}"
                    )
                    return {trusted_role}
            if record and record.role:
                from infrastructure.db.subagent_repository import _normalize_role, _get_known_roles_from_config
                known_roles = _get_known_roles_from_config()
                normalized = _normalize_role(record.role, known_roles)
                logger.info(
                    f"CALLER ROLES (agent_id): agent_id={agent_id}, role={normalized}"
                )
                return {normalized}
            # record/role無し → 空set（安全側フォールバック）
            return set()
        except Exception as e:
            logger.warning(f"Failed to get caller roles via agent_id: {e}")
            return set()

    # agent_id不在 = leader（is_leader_tool_use()と一貫した判定: issue_8118）
    logger.info("CALLER ROLES: agent_id不在 → leader")
    return {"leader"}
